/********************************************************/
/*      BASIC IMMAGE MODULE                             */
/* This module contains all the basic routines that	*/
/* are used for storing the 3D multi spect. stuff	*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/
/*      Ideas      :    Frank Fueten, Scott Goodchild   */
/*      Programming:    Vladimir Tucakov                */
/********************************************************/

#include <gl/gl.h>
#include <stdio.h>
#include <stdlib.h>
#include <gl/device.h>
     #include <sys/types.h>
     #include <sys/stat.h>
     #include <fcntl.h>

#include "image.h"
#include "basic_imm.h"
#include <sys/types.h>
#include <malloc.h>
#include "types.h"


int density=8;
static int y, xsize, ysize;
static int z, zsize;

extern screenT screen;


/********************************************************/
/* Gets the value of the pixel at the given r,g,b	*/
/* coordinates from the given space. For the time being */
/* it uses complicated bitwise logic to return only 	*/
/* zero of one. In the future it could be changed to 	*/
/* return a larger number, this however would require 	*/
/* change of the space data sturcture.			*/
/********************************************************/
char get_from_space(thetype space, short r, short g, short b)
{
return(space[r][g][b]);
/*
char temp =1;
	if ((space[r][g][b/DEVIDER] & (temp << (b % DEVIDER))) > 0)
		return(1);
	else
		return(0);
*/
}

void add_to_space(thetype space, short r, short g, short b, char value)
{
	if ((int)space[r][g][b] + (int)value < 255)
		space[r][g][b] += value;
	else 
		space[r][g][b] = 255;
}

/********************************************************/
/* Puts the given value at the specified rgb position	*/
/* in the given 3D space. For now it uses conplicated 	*/
/* bitwise operations and can store only 0 or 1. In the	*/
/* future it can be changed to store higher numbers	*/
/********************************************************/
void put_to_space(thetype space, short r, short g, short b, char value)
{
space[r][g][b] = value;
/*
char temp ;
temp = (char)1;

	temp = temp << (b % DEVIDER);
	space[r][g][b/DEVIDER] &= (char)~temp;
	space[r][g][b/DEVIDER] |= (value << (b % DEVIDER));
*/
}


/********
ALLOCATES STORAGE FOR THE 3D SPACE
*********/ 
void make_space(thetype *space) 
{
	*space = calloc(TOT_SPACE_SIZE,sizeof(twotype));
	if (*space == NULL)
	{
	       fprintf(stderr,"Memory allocation problem. EXITING PROGRAM !\n");
	       exit(0);
	}
}


/********************************************************/
/* Deallocates the given 3D space			*/
/********************************************************/
void kill_space(thetype space)
{
int i;
	if (space != NULL)
	for(i=0; i<TOT_SPACE_SIZE; i++)
	{
		free(space[i]);
	}
}

/********************************************************/
/* NOT USED, KEPT FOR HISTORIC VALUE			*/
/********************************************************/
/*
int popular_point(thetype space,char r, char g, char b, int size)
{
int i,j,k,counter=0;
int  rmin = (int)r-size , rmax = (int)r+size,
     gmin = (int)g-size , gmax = (int)g+size,
     bmin = (int)b-size , bmax = (int)b+size,
     brk = TRUE;

if (rmin < 0) rmin = 0;
if (gmin < 0) gmin = 0;
if (bmin < 0) bmin = 0;
if (rmax >= TOT_SPACE_SIZE) rmax = TOT_SPACE_SIZE -1;
if (gmax >= TOT_SPACE_SIZE) gmax = TOT_SPACE_SIZE -1;
if (bmax >= TOT_SPACE_SIZE) bmax = TOT_SPACE_SIZE -1;

for (i=rmin; (i<=rmax) && (brk == TRUE); i++)
        for (j=gmin; (j<=gmax) && (brk == TRUE); j++)
                for (k=bmin; (k<=bmax) && (brk == TRUE); k++)
                {
                        if (!(i == r && j == g && k == b) &&
                        get_from_space(space,(char)i,(char)j,(char)k) > 0)
                        {
                                counter++;
                        }
                }
if (counter<=3) brk = FALSE;
return (brk);
}
*/

void ms_threshold(thetype space,int thresh)
{
int i,j,k,count=0;

for(i=0; i<TOT_SPACE_SIZE; i++)
        for(j=0; j<TOT_SPACE_SIZE; j++)
                for(k=0; k<TOT_SPACE_SIZE; k++)
                {

		    if(get_from_space(space,i,j,k) <= thresh 
				&& get_from_space(space,i,j,k) > 0)
		    {
		        count += get_from_space(space,i,j,k);
			put_to_space(space,i,j,k,0);
		    }
		}

fprintf(stderr,"THRESHOLDING : KILLED %d POINTS !!!!\n\n",count);

}


/********************************************************/
/* Returns weather the a point has "people" number of 	*/
/* points in the place*place*place space around it	*/
/* in the given "space"					*/
/********************************************************/
int lonely_point(thetype space,char r, char g, char b, short people, int place)
{
int i,j,k,counter=0;
int  rmin = (int)r-place , rmax = (int)r+place, 
     gmin = (int)g-place , gmax = (int)g+place, 
     bmin = (int)b-place , bmax = (int)b+place,
     brk = TRUE;
	
if (rmin < 0) rmin = 0;
if (gmin < 0) gmin = 0;
if (bmin < 0) bmin = 0;
if (rmax >= TOT_SPACE_SIZE) rmax = TOT_SPACE_SIZE -1;
if (gmax >= TOT_SPACE_SIZE) gmax = TOT_SPACE_SIZE -1;
if (bmax >= TOT_SPACE_SIZE) bmax = TOT_SPACE_SIZE -1;

for (i=rmin; (i<=rmax) && (brk == TRUE); i++) 
	for (j=gmin; (j<=gmax) && (brk == TRUE); j++)
		for (k=bmin; (k<=bmax) && (brk == TRUE); k++)
		{
			if (!(i == r && j == g && k == b) && 
			get_from_space(space,(char)i,(char)j,(char)k) > 0) 
			{
				counter++;
				if (counter > people)
						brk = FALSE;
			}
		}
return (brk);
}

/*******
DELETES ALL LONELY POOINTS
********/
/********************************************************/
/* "space2" is filled with all points that are not	*/
/* "lonely" in "space", for the given neighbourhood	*/
/* "place" and number of neighbours "people"		*/
/* "space" is not changed.				*/
/********************************************************/
int kill_lonely(thetype space, thetype space2, short people, short place)
{
int i,j,k,counter=0;
long X[3];

fprintf(stderr,"\nFILTERING space... Please wait...\n");
for (i=0; i<TOT_SPACE_SIZE; i++)
{
fprintf(stderr,"%d/255 DONE...\r",i);
        for(j=0; j<TOT_SPACE_SIZE; j++)
        {
                for(k=0; k<TOT_SPACE_SIZE; k++)
                {
                        if (get_from_space(space,i,j,k)>0 && 
				lonely_point(space, i,j,k,people,place)!=TRUE)
                        {
				counter++;
                                put_to_space(space2,i,j,k,
					get_from_space(space,i,j,k));
                        }
			
                }
        }
}
return(counter);
}





/********************************************************/
/* The given 3D space is filled according to the 	*/
/* given domain. The 2D image is the global variable	*/
/* screen, and the domain specifies weather the 3D 	*/
/* space is filled with, min, max, average or positions	*/
/********************************************************/
int fill_space(thetype space, char domain)
{
int i,j;
short r,g,b;
long count = 0,count2 = 0;

float ratio;


if (domain == DOM_AVER)
	ratio = (255.0*(float)screen.no_frames)/(float)TOT_AXIS_SIZE;
else 
	ratio = 255.0/TOT_AXIS_SIZE;

fprintf(stderr,"ratio = %f\n",ratio);


fprintf(stderr,"\nFILLING the 3D space\n");

	for(i=0; i<screen.xsize; i++)
	{
fprintf(stderr,"%d/255 Done.\r",i);
		for(j=0; j<screen.ysize; j++)
		{
count++;
		if (domain == DOM_AVER && get_from_space(space, 
		  r = (short)(((float)screen.scr[i][j].tot[SR])/ratio),
		  g = (short)(((float)screen.scr[i][j].tot[SG])/ratio),
		  b = (short)(((float)screen.scr[i][j].tot[SB])/ratio)) == 0)
		     {
			count2++;
		     }

		if (domain == DOM_MINV && get_from_space(space, 
		  r = (short)(((float)screen.scr[i][j].min[SR])/ratio),
		  g = (short)(((float)screen.scr[i][j].min[SG])/ratio),
		  b = (short)(((float)screen.scr[i][j].min[SB])/ratio)) == 0)
		     {
			count2++;
		     }

		if (domain == DOM_MINP && get_from_space(space, 
		  r = (short)(((float)screen.scr[i][j].min_pos[SR])/ratio),
		  g = (short)(((float)screen.scr[i][j].min_pos[SG])/ratio),
		  b = (short)(((float)screen.scr[i][j].min_pos[SB])/ratio))==0)
		     {
			count2++;
		     }

		if (domain == DOM_MAXV && get_from_space(space, 
		  r = (short)(((float)screen.scr[i][j].max[SR])/ratio),
		  g = (short)(((float)screen.scr[i][j].max[SG])/ratio),
		  b = (short)(((float)screen.scr[i][j].max[SB])/ratio))==0)
		     {
			count2++;
		     }

		if (domain == DOM_MAXP && get_from_space(space, 
		  r = (short)(((float)screen.scr[i][j].max_pos[SR])/ratio),
		  g = (short)(((float)screen.scr[i][j].max_pos[SG])/ratio),
		  b = (short)(((float)screen.scr[i][j].max_pos[SB])/ratio))==0)
		     {
			count2++;
		     }
/*
fprintf(stderr,"%d   %d  %d  %d\n",j,r,g,b);
*/
		add_to_space(space,r,g,b,1);

		}
	}

fprintf(stderr,"the image has %d points\n",count);
fprintf(stderr,"the 3D space was filled with %d points\n",count2);
return(count2);
}



/********************************************************/
/* An enhansement of the GL routine that defines a 	*/
/* point in 3D. This one takes in account the constants	*/
/* that define the size of the 3D multi-spect space	*/
/********************************************************/
void put_point(float a[3])
{
float b[3];
	b[0] = a[0]-RGB_AXIS_SIZE/2;
	b[1] = a[1]-RGB_AXIS_SIZE/2;
	b[2] = a[2]-RGB_AXIS_SIZE/2;
	v3f(b);
}

/********************************************************/
/* This routine fills the 3D GL space with the info 	*/
/* stored in the 3D m.s. space.				*/
/********************************************************/
void put_RGB_2(thetype space)
{
int i,j,k,count=0;
float X[3];

X[0] = 0; X[1] = 0; X[2] = 0;
color(WHITE);
bgnpoint();
for(i=0; i<TOT_SPACE_SIZE; i++)
{
        for(j=0; j<TOT_SPACE_SIZE; j++)
	{
                for(k=0; k<TOT_SPACE_SIZE; k++)
		{
                        if(get_from_space(space,(char)i,(char)j,(char)k) > 0)
			{
count++;
				X[0] = (float)i;
				X[1] = (float)j;
				X[2] = (float)k;
				put_point(X);
			}
		}
	}
}
endpoint();
}

/*
put_RGB_2()
{
int i,j;
long X[3];

X[0] = 0; X[1] = 0; X[2] = 0;
color(WHITE);
	bgnpoint();
		for(i=0; i<ysize; i+=density)
		{
		for(j=0; j<xsize; j+=density)
		{
			X[0] = *(rbuff+j+(i*xsize));
			X[1] = *(gbuff+j+(i*xsize));
			X[2] = *(bbuff+j+(i*xsize));
			put_point(X);
		}
		}
	endpoint();
}
*/


/********************************************************/
/* Loads an RGB file given by argv into the buffer 	*/
/* given at the location data				*/
/********************************************************/
void load_rgb_image(char *argv, char **data)
{
    char *temp;
    IMAGE *image;
    int xsize, ysize, x, y;
    short rbuf[XSIZE*YSIZE];
    short gbuf[XSIZE*YSIZE];
    short bbuf[XSIZE*YSIZE];

    if( (image=iopen(argv ,"r")) == NULL ) 
    {
        fprintf(stderr,"readimg: can't open input file \n");
        exit(1);
    }
    xsize = image->xsize;
    ysize = image->ysize;
    zsize = image->zsize;
    if(zsize<3) 
    {
        fprintf(stderr,"readimg: this is not an RGB image file\n");
        exit(1);
    }

    for(y=0; y<ysize; y++) 
    {
        getrow(image,rbuf+(y*xsize),ysize - 1- y,0);
        getrow(image,gbuf+(y*xsize),ysize - 1- y,1);
        getrow(image,bbuf+(y*xsize),ysize - 1- y,2);
    }
    iclose(image);

  *data = calloc(xsize*ysize*4,1);
  if (data == NULL)
  {
        fprintf(stderr,"Memory allocation problem. EXITING PROGRAM !\n");
        exit(0);
  }
  temp = *data;
  for (y=0;y<ysize;y++) 
  {
/*
fprintf(stderr," in there %d\n",y);
*/    
    for(x=0;x<xsize;x++) 
    {
      *(temp+1) = bbuf[y*xsize + x];
      *(temp+2) = gbuf[y*xsize + x];
      *(temp+3) = rbuf[y*xsize + x];
      temp += 4;
    }
  }
}

/********************************************************/
/* Saves the information given at address data into	*/
/* file ginven by filename. Size of the image must be	*/
/* specified.						*/
/********************************************************/
void save_rgb_image(char filename[], char *data, int xsize, int ysize)
{
  IMAGE *image;
  short rbuf[XSIZE*YSIZE];
  short gbuf[XSIZE*YSIZE];
  short bbuf[XSIZE*YSIZE];
  int x, y, z;
  image = iopen(filename, "w", RLE(1), 3, xsize, ysize, 3);
  for (y=ysize-1; y>=0; y--) 
  {
    for(x=0;x<xsize;x++) 
    {
      bbuf[x] = *(data+1);
      gbuf[x] = *(data+2);
      rbuf[x] = *(data+3);
      data += 4;
    }
    putrow(image, rbuf, y, 0);
    putrow(image, gbuf, y, 1);
    putrow(image, bbuf, y, 2);

  }
  iclose(image);
}


four_to_three(char *four, char *three)
{
int x,y;

	four++;
	for (y=0; y<YSIZE; y++)
	for (x=0; x<XSIZE; x++)
	{
/*
		*three = valat(four,x,y,R);
		three++;
		*three = valat(four,x,y,G);
		three++;
		*three = valat(four,x,y,B);
		three++;
*/
		memcpy(three,four,3);
		three += 3;
		four += 4;
	}
}


void save_raw_rgb(char filename[], char *data, int xsize, int ysize)
{
int file, x, y;
char *small_data;

	small_data = calloc(xsize*ysize,3);
	if (small_data == NULL)
	{
            fprintf(stderr,"Memory allocation problem. EXITING PROGRAM !\n");
            exit(0);
	}

	four_to_three(data, small_data);
	
	file = open(filename, O_WRONLY|O_CREAT);
fprintf(stderr,"%d %d %d\n", small_data[11], small_data[10002], small_data[1003]);
	write(file, small_data, 3*xsize*ysize);
fprintf(stderr,"%d %d %d\n",valat(data,10,10,R),
			valat(data,10,10,G),valat(data,10,10,B));

	close(file);
	free(small_data);
}


three_to_four(char *three, char *four)
{
int x,y;

        for (y=0; y<YSIZE; y++)
        for (x=0; x<XSIZE; x++)
	{
		*four = 0;
		four++;
		memcpy(four, three, 3);
		four += 3;
		three += 3;
	}

}


/****************************************/
/* space for data should be allocated ! */
/****************************************/
void load_raw_rgb(char filename[], char *data, int xsize, int ysize)
{
int file,i,j;
char *small_data,*temp;

	small_data = calloc(xsize*ysize,3);
	if (small_data == NULL)
	{
             fprintf(stderr,"Memory allocation problem. EXITING PROGRAM !\n");
             exit(0);
	}

	file = open(filename, O_RDONLY);

	read(file, small_data, 3*xsize*ysize);
	three_to_four(small_data, data);

	close(file);
	free(small_data);	
}
