/********************************************************/
/*      BOXING MODULE    	                        */
/* 	This module contains routines that handle the	*/
/*	the definition of thresholding values for 	*/
/*	multispectral analysis.				*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/
/*      Ideas      :    Frank Fueten, Scott Goodchild   */
/*      Programming:    Vladimir Tucakov                */
/********************************************************/

#include <gl/gl.h>
#include <stdio.h>
#include <malloc.h>
#include "types.h"
#include "basic_imm.h"
#include "twoD.h"
#include "forms.h"
#include "gv_forms.h"
#include "boxing.h"

#define MAN_SIZE 130
#define BOX_SIZE 25
#define BOX_START 128
#define R_MAX 1
#define R_MIN 2
#define G_MAX 3
#define G_MIN 4
#define B_MAX 5
#define B_MIN 6
#define B_UP 1
#define B_DOWN 2


extern twotype *space1;
static int sp_1, sp_2;


static    int y, xsize, ysize;
static    int z, zsize;
extern int density;

extern pointT *disp_arr;
extern disp_size;
extern short box;
extern short old_r_min,old_r_max,old_g_min,old_g_max,old_b_min,old_b_max;

box_T *boxing;
box_T *curr_box;

unsigned short man_color = RED;


/********************************************************/
/* The box data structure is allocated and initial 	*/
/* values assigned. The starting position and the size	*/
/* are predefined constants.				*/
/********************************************************/
box_T *init_box(unsigned short color)
{
box_T *temp;

temp = calloc(1,sizeof(box_T));

temp->color= color;

temp->r_min= BOX_START-BOX_SIZE; 
temp->r_max = BOX_START+BOX_SIZE;
temp->g_min= BOX_START-BOX_SIZE; 
temp->g_max = BOX_START+BOX_SIZE;
temp->b_min= BOX_START-BOX_SIZE; 
temp->b_max = BOX_START+BOX_SIZE;

set_box_counters(temp);
temp->prev = NULL;
temp->next = NULL;


return(temp);
}

/********************************************************/
/* Seting the FORMS counter values			*/
/********************************************************/
void set_box_counters(box_T *temp)
{
fl_set_counter_value(ms_r_min, old_r_min = temp->r_min);
fl_set_counter_value(ms_r_max, old_r_max = temp->r_max);
fl_set_counter_value(ms_g_min, old_g_min = temp->g_min);
fl_set_counter_value(ms_g_max, old_g_max = temp->g_max);
fl_set_counter_value(ms_b_min, old_b_min = temp->b_min);
fl_set_counter_value(ms_b_max, old_b_max = temp->b_max);
}

/********************************************************/
/* The GL based routine that draws the tresholding box	*/
/********************************************************/
void put_little_box(box_T *box)
{
float X[3];
char i;
Linestyle ls;

        setlinestyle(1);
        color((unsigned short) box->color);
        X[BR]=box->r_min;X[BG]=box->g_min;X[BB]=box->b_min;
        for(i=0; i<=1; i++)
        {
                bgnline();
                        put_point(X);
                        X[BG]=box->g_max;
                        put_point(X);
                        X[BR]=box->r_max;
                        put_point(X);
                        X[BG]=box->g_min;
                        put_point(X);
                        X[BR]=box->r_min;
                        put_point(X);
                endline();
                X[BB]=box->b_max;
        }
        bgnline();
                X[BB]=box->b_min;
                put_point(X);
                X[BB]=box->b_max;
                put_point(X);
        endline();
        X[BG]=box->g_max;
        bgnline();
                X[BB]=box->b_min;
                put_point(X);
                X[BB]=box->b_max;
                put_point(X);
        endline();
        X[BR]=box->r_max;
        bgnline();
                X[BB]=box->b_min;
                put_point(X);
                X[BB]=box->b_max;
                put_point(X);
        endline();
        X[BG]=box->g_min;
        bgnline();
                X[BB]=box->b_min;
                put_point(X);
                X[BB]=box->b_max;
                put_point(X);
        endline();
        deflinestyle(0xffff,ls);
        setlinestyle(0);

}


/********************************************************/
/* Changes the values of the box data structure based	*/
/* on the current value of the FORMS counters		*/
/********************************************************/
void b_move(box_T *box)
{

	box->r_min = (short) fl_get_counter_value(ms_r_min);
	box->r_max = (short) fl_get_counter_value(ms_r_max);

	box->g_min = (short) fl_get_counter_value(ms_g_min);
	box->g_max = (short) fl_get_counter_value(ms_g_max);

	box->b_min = (short) fl_get_counter_value(ms_b_min);
	box->b_max = (short) fl_get_counter_value(ms_b_max);
}


/********************************************************/
/* Adds a box the linked list of boxes and assignes it	*/
/* a color						*/
/********************************************************/
void new_box(box_T **curr_box, unsigned short color)
{
box_T *temp;

temp = init_box(color);


if (*curr_box != NULL)
{
	(*curr_box)->next = temp;
	temp->prev = *curr_box;
	*curr_box = temp;
}
else
	*curr_box = temp;
}



/********************************************************/
/* Self explanatory					*/
/********************************************************/
void set_box_color(box_T *curr_box,unsigned short color)
{
	curr_box->color = color;
}



/********************************************************/
/* Changes the color of the pixels in the display array	*/
/* that belong to the given box. The poixels are colored*/
/* to the given color					*/
/********************************************************/
void blank_box(box_T *l_box, pointT *disp, long size,unsigned short color)
{
long i;

for(i=0; i<size; i++)
{

	if (in_box(l_box,(char)disp[i].val[0],(char)disp[i].val[1],
		(char)disp[i].val[2])== TRUE)
	{
		disp[i].color = (char) color;
	}
}

}

/********************************************************/
/* Deletes the box from the linked list of boxes	*/
/********************************************************/
int del_box(box_T **curr, unsigned short *color)
{
box_T *temp;

if((*curr)->prev != NULL)
{
	temp = (*curr);
	(*curr) = temp->prev;
	(*curr)->next = temp->next;
	if (temp->next != NULL)
		(temp->next)->prev = (*curr);
	free(temp);
	*color = (*curr)->color;
	return(TRUE);
}
else if((*curr)->next != NULL)
{
	temp = (*curr);
	(*curr) = temp->next;
	if(temp->prev != NULL)
		(temp->prev)->next = temp->next;
	(*curr)->prev = temp->prev;
	free(temp);
	*color = (*curr)->color;
	return(TRUE); 
}
return(FALSE);
} 



/********************************************************/
/* saves the given box linked list to the file with	*/
/* the given name					*/
/********************************************************/
void save_box(char *string, box_T *box)
{
FILE *file;
char temp[80];

sprintf(temp,"%s.box",string);
if ((file = fopen(temp,"w")) == NULL)
{
        fprintf(stderr, "error opening file\n");
        exit(0);
}

while(box != NULL)
{
	fwrite(&box->color,sizeof(box->color),1,file);
	fwrite(&box->r_min,sizeof(box->r_min),6,file);
	box = box->next;
}
fclose(file);
}




/********************************************************/
/* Loads the box data structure from the given file	*/
/* name and allocates the space for it			*/
/********************************************************/
void load_box(char *string, box_T **box, box_T **current)
{
FILE *file;
box_T temp;
char t[80];

sprintf(t,"%s.box",string);
if ((file = fopen(t,"r")) == NULL)
{
        fprintf(stderr, "error opening file\n");
        exit(0);
}
while(1)
{
	fread(&temp.color,sizeof(temp.color),1,file);
	if (feof(file) != 0)
		break;
	fread(&temp.r_min,sizeof(temp.r_min),6,file);

	new_box(current,temp.color);
	(*current)->r_min = temp.r_min;
	(*current)->r_max = temp.r_max;
	(*current)->g_min = temp.g_min;
	(*current)->g_max = temp.g_max;
	(*current)->b_min = temp.b_min;
	(*current)->b_max = temp.b_max;
	(*current)->color = temp.color;
	if (*box == NULL)
		*box = *current;
}
fclose(file);
}

/********************************************************/
/* Deallocates the storage space for the boxing info.	*/
/********************************************************/
void kill_boxes(box_T **box)
{
box_T *temp;

	while(*box != NULL)
	{
		temp = *box;
		*box = (*box)->next;
		free(temp);
	}
}
