#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "forms.h"
#include "gv_forms.h"
#include "camera.h"
#include "stage.h"

extern char *dataPtr;
extern short max_caxis;

/********************************************************/
/*Calibrate the camera					*/
/********************************************************/
void cal_cam_proc(FL_OBJECT *n, long m)
{
short i, x, y;
Boolean rotate_flag;
long camera_r = 0;
long camera_g = 0;
long camera_b = 0;
FILE *file;
char *filename;

filename = fl_show_file_selector("File Name To Save","","*.dat","");

if((file = fopen(filename, "w")) == NULL)
{
	fl_show_message("UNABLE TO OPEN FILE!","","");
}

else
{
	open_video();
	rotate_flag = fl_get_button(c_which_polars_but); 
	for(i = 0; i <= 200; i++)
	{
		get_one_image();
		dump_image(dataPtr);

		for(x = 200; x < 300; x++)
		{
			for(y = 200; y < 300; y++)
			{
				camera_r += (long)valat(dataPtr,x,y,R);
				camera_g += (long)valat(dataPtr,x,y,G);
				camera_b += (long)valat(dataPtr,x,y,B);
			}
		}
	
		camera_r /= 10000;
		camera_g /= 10000;
		camera_b /= 10000;

		fprintf(file,"%ld\t%ld\t%ld\n",camera_r, camera_g, camera_b);
	
		if( (i != 200)&&(rotate_flag == 0) )
			rotate_polarizer(UPPER_POLAR, 1);

		if( (i != 200)&&(rotate_flag == 1) )
			rotate_polarizer(BOTH_POLARS, 1);

	}
	fclose(file);

	if(rotate_flag == 0)
		rotate_polarizer(UPPER_POLAR, -200);

	else
		rotate_polarizer(BOTH_POLARS, -200);

	close_video();
}
}

/********************************************************/
/*Calibrate quartz c-axes				*/
/********************************************************/
void cal_caxes_proc(FL_OBJECT *n, long m)
{
short i, j, x, y, box = 20;
char *tmp;
int known_plunge;
long caxis;
int array[200][3];
FILE *file;
char *filename;

filename = fl_show_file_selector("File Name To Save","","*.dat","");
file = fopen(filename, "w");

tmp = fl_get_input(caxes_plunge_input);
sscanf(tmp, "%d", &known_plunge);

for(i = 0; i < 200; i++)
{
	for(j = 0; j < 3; j++)
	{
		array[i][j] = 0;
	}
}

if(known_plunge < 0 || known_plunge > 90)
{
	fl_show_message("PLUNGE MUST BE BETWEEN 0 AND 90!","","");
}

else
{
	fprintf(stderr,"Max = %d\n", max_caxis);
	open_video();
	for(i = 0; i <= 200; i++)
	{
		get_one_image();
		dump_image(dataPtr);

		max_caxis = 0;
		for(x = box; x < XSIZE - box; x++)
		{
			for(y = box; y < YSIZE - box; y++)
			{
				caxis = (long)(valat(dataPtr,x,y,R)+valat(dataPtr,x,y,G)+
						valat(dataPtr,x,y,B));
				
				if(caxis > max_caxis)
				{
					array[i][0] = (int)valat(dataPtr,x,y,R);
					array[i][1] = (int)valat(dataPtr,x,y,G);
					array[i][2] = (int)valat(dataPtr,x,y,B);
					max_caxis = caxis;
				}	
			}
		}

		fprintf(stderr,"%d\t%d\t%d\t%d\n",i, array[i][0], array[i][1], array[i][2]);
		fprintf(file,"%d\t%d\t%d\t%d\n",i, array[i][0], array[i][1], array[i][2]);

		if(i != 200)
			rotate_polarizer(BOTH_POLARS, 1);
		/*sleep(1);*/
	}
	rotate_polarizer(BOTH_POLARS, -200);
	close_video();	
	fprintf(stderr,"Max = %d\n", max_caxis/3);
	fclose(file);
}
}

/********************************************************/
/*Real time sampling					*/
/********************************************************/
void real_time_sample_proc(FL_OBJECT *n, long m)
{
short x, y;
unsigned int sumr, sumg, sumb;
FL_OBJECT *obj;
 
fl_hide_object(progress);
fl_show_object(go);
fl_show_form(samp_info,FL_PLACE_CENTER,FALSE,NULL);
do obj = fl_do_forms(); while (obj != s_info_go );
fl_hide_object(go);

fl_set_object_label(progres_text,"Real Time Sampling");
fl_show_object(progress);

open_video();

get_one_image();
dump_image(dataPtr);

while(TRUE)
{
	/* CHECKING FOR INTERRUPTIONS */
	if(fl_check_forms() == s_info_stop)
		break;

	get_one_image();
	dump_image(dataPtr);
	
	sumr = 0;
	sumg = 0;
	sumb = 0;
	for(x = 315; x < 325; x++)
	{
		for(y = 235; y < 245; y++)
		{
			sumr += (int)(valat(dataPtr, x, y, R));
			sumg += (int)(valat(dataPtr, x, y, G));
			sumb += (int)(valat(dataPtr, x, y, B));
		}
	}
	sumr /= 100;
	sumg /= 100;
	sumb /= 100;
	
	fprintf(stderr,"\nRed = %d\tGreen = %d\tBlue = %d\n", sumr, sumg, sumb);
}
close_video();
fl_hide_object(progress);
fl_hide_form(samp_info);
}

/********************************************************/
/*closes camera dialog.					*/
/********************************************************/
void camera_done_proc(FL_OBJECT *n, long m)
{
fl_hide_object(camera_grp);
fl_show_object(main_grp);
}
 
