#include <gl/gl.h>
#include <gl/device.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "forms.h"
#include "gv_forms.h"
#include "types.h"
#include "stage.h"
#include "edge.h"
 
extern char *dataPtr;
extern char two_dump[XSIZE][YSIZE][3];
extern screenT screen;


static short xzoom_zero, yzoom_zero;
static current_image_flag = E_EDGE;

char zoom_buffer[XSIZE][YSIZE][3];

short zoom_flag = 0, edit_flag = 0;

short protected[XSIZE][YSIZE];
long edge_win = -1;

/****************************************************************/
/*Opens the edge window and dumps the current edges into it.	*/	
/****************************************************************/
void start_edge()
{
prefsize(XSIZE, YSIZE);
edge_win = winopen("EDGE WINDOW");
RGBmode();
pixmode(PM_TTOB, 1);
gconfig();
edge_init_colours();
put_edge_pixels();
dump_image(two_dump);
}

/****************************************************************/
/*Initilaizes the colour selection buttons in the edge menu.	*/
/****************************************************************/
void edge_init_colours()
{
fl_set_button(edge_r, 0);
fl_set_button(edge_g, 0);
fl_set_button(edge_b, 0);
}

/****************************************************************/
/*Initializes forward and reverse threshold counters.		*/
/****************************************************************/
threshold_init()
{
fl_set_counter_bounds(threshold_image, 0, 255);
fl_set_counter_step(threshold_image, 1, 10);
fl_set_counter_precision(threshold_image, 0);
fl_set_counter_value(threshold_image, 0);

fl_set_counter_bounds(negative_threshold_image, 0, 255);
fl_set_counter_step(negative_threshold_image, 1, 10);
fl_set_counter_precision(negative_threshold_image, 0);
fl_set_counter_value(negative_threshold_image, 255);
}

/****************************************************************/
/*Dumps the initial edges into the working memory location. 	*/
/*Also fills the undo array with this initial data.		*/
/****************************************************************/
void put_edge_pixels()
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, screen.scr[x][y].complete_edge);
		assval(two_dump, x, y, G, screen.scr[x][y].complete_edge);
		assval(two_dump, x, y, B, screen.scr[x][y].complete_edge);
	}
}

}

/****************************************************************/
/*Scales down the array carrying the gradient data obtained by	*/
/*sampling to a range 0 - 255. 					*/
/****************************************************************/
void edge_scale_down(unsigned long overflow_pixel_counter[XSIZE][YSIZE])
{
long x, y;
long dummy, num_pixels = 311040;
unsigned long max_pixel_overflow = 0, num_times[15300], total_sum = 0;
unsigned long  min_pixel_overflow = 255, cut_off_value_top, cut_off_value_bottom, a;

for(x = 5; x < XSIZE - 5; x++)
{
	for(y = 5; y < YSIZE - 5; y++)
	{
		if(overflow_pixel_counter[x][y] > max_pixel_overflow)
		{
			max_pixel_overflow = overflow_pixel_counter[x][y];
		}

		if(overflow_pixel_counter[x][y] < min_pixel_overflow)
		{
			min_pixel_overflow = overflow_pixel_counter[x][y];
		}
	}
}

for(x = 0; x <= max_pixel_overflow; x++)
{
	num_times3[x] = 0;
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		num_times3[overflow_pixel_counter[x][y]]++;
	}		
}

cut_off_value_bottom = min_pixel_overflow;

for(a = 0; a <= max_pixel_overflow; a++)
{
	total_sum = num_times3[a] + total_sum;
	
	if(total_sum <= 0.95 * num_pixels)  
	{
		cut_off_value_top = a;/*max_pixel_overflow*/
	}
}

for(x = 5; x < XSIZE - 5; x++)
{
	for(y = 5; y < YSIZE - 5; y++)
	{
		dummy = overflow_pixel_counter[x][y];
		dummy = dummy - cut_off_value_bottom;
		if(dummy < 0)
		{
			overflow_pixel_counter[x][y] = 0;
		}
	}
}

for(x = 5; x < XSIZE - 5; x++)
{
	for(y = 5; y < YSIZE - 5; y++)
	{
		if(overflow_pixel_counter[x][y] > (cut_off_value_top - cut_off_value_bottom))
		{
			overflow_pixel_counter[x][y] = (cut_off_value_top - cut_off_value_bottom);
		}
	}
}

for(x = 5; x < XSIZE - 5; x++)
{
	for(y = 5; y < YSIZE - 5; y++)
	{
		screen.scr[x][y].sample_edge = (overflow_pixel_counter[x][y] * 255)
						/ (cut_off_value_top - cut_off_value_bottom);
		screen.scr[x][y].complete_edge = screen.scr[x][y].sample_edge;
	}
}

}

/****************************************************************/
/*Passes a 5x5 median filter over the current image in the edge	*/
/*window -- should be a grey level image.			*/
/****************************************************************/
void median_filter_image_proc(FL_OBJECT *n, long m)
{
short x, y, xpos, ypos, a, b, ir;
char check_r;
char pixel_max_r[XSIZE][YSIZE];
char new_r[XSIZE][YSIZE], median_r[25];

for (x = 0; x < XSIZE; x++) 
{
	for (y = 0; y < YSIZE; y++)	
	{
		new_r[x][y] = 0;
		pixel_max_r[x][y] = valat(two_dump, x, y, R);
	}
}

for (x = 0; x < XSIZE; x++) 
{
	for (y = 0; y < YSIZE; y++)	
	{
		a = 0;
		for(xpos = x - 2; xpos <= x + 2; xpos++)
		{
			for(ypos = y - 2; ypos <= y + 2; ypos++)
			{
				if( (xpos > 0) && (xpos < XSIZE)
				&&(ypos > 0) && (ypos < YSIZE) )
				{
					median_r[a] = pixel_max_r[xpos][ypos];
					a++;
				}
			}
		}
		
		for(b = 0; b < a; b++)
		{
			check_r = median_r[b];
			
			ir = b - 1;
			
			while( (ir >= 0) && (median_r[ir] > check_r) )
			{
				median_r[ir + 1] = median_r[ir];
				ir--;
			}
			
			median_r[ir + 1] = check_r;
		}
		new_r[x][y] = median_r[a / 2];	
	}
}

for (x = 0; x < XSIZE; x++) 
{
	for (y = 0; y < YSIZE; y++)	
	{
		assval(two_dump, x, y, R, new_r[x][y]);
		assval(two_dump, x, y, G, new_r[x][y]);
		assval(two_dump, x, y, B, new_r[x][y]);
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Passes a 3x3 mean filter over the current image in the edge	*/
/*window -- should be a grey level image.			*/
/****************************************************************/
void mean_filter_image_proc(FL_OBJECT *n, long m)
{
short x, y, xpos, ypos;
short blur_x = 1, blur_y = 1;
short blur = 9;
unsigned long av_val_red[XSIZE][YSIZE];

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		av_val_red[x][y] = 0;

		for(xpos = x - blur_x; xpos <= x + blur_x; xpos++)
		{
			for(ypos = y - blur_y; ypos <= y + blur_y; ypos++)
			{
				if( (xpos > 0) && (xpos < XSIZE)
				&&(ypos > 0) && (ypos < YSIZE) )
				{
					av_val_red[x][y] = av_val_red[x][y] + valat(two_dump, xpos, ypos, R);
				}
			}
		}
	}
}

for(x = blur_x; x < XSIZE - blur_x; x++)
{
	for(y = blur_y; y < YSIZE - blur_y; y++)
	{
		assval(two_dump, x, y, R, av_val_red[x][y] / blur);
		assval(two_dump, x, y, G, av_val_red[x][y] / blur);
		assval(two_dump, x, y, B, av_val_red[x][y] / blur);
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Passes a 7x7 gaussian filter over the current image in the 	*/
/*edge window -- should be a grey level image.			*/
/****************************************************************/
void gaussian_filter_image_proc(FL_OBJECT *n, long m)
{
short x, y, xpos, ypos;
char value_r[XSIZE][YSIZE];
char pixel_value_r[XSIZE][YSIZE];
unsigned int c1_r, c2_r, c3_r;
unsigned int c4_r, c5_r, c6_r, c7_r;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		value_r[x][y] = valat(two_dump, x, y, R);
		pixel_value_r[x][y] = 0;
	}
}

for(x = 3; x < XSIZE - 3; x++)
{
	for(y = 3; y < YSIZE - 3; y++)
	{
		c1_r = value_r[x-3][y-3] + value_r[x-3][y-2] + 2*value_r[x-3][y-1] +
		     2*value_r[x-3][y] + 2*value_r[x-3][y+1] + value_r[x-3][y+2] + value_r[x-3][y+3];

		c2_r = value_r[x-2][y-3] + 2*value_r[x-2][y-2] + 2*value_r[x-2][y-1] +
		     4*value_r[x-2][y] + 2*value_r[x-2][y+1] + 2*value_r[x-2][y+2] + value_r[x-2][y+3];
		
		c3_r = 2*value_r[x-1][y-3] + 2*value_r[x-1][y-2] + 4*value_r[x-1][y-1] +
		     8*value_r[x-1][y] + 4*value_r[x-1][y+1] + 2*value_r[x-1][y+2] + 2*value_r[x-1][y+3];
		
		c4_r = 2*value_r[x][y-3] + 4*value_r[x][y-2] + 8*value_r[x][y-1] +
		     16*value_r[x][y] + 8*value_r[x][y+1] + 4*value_r[x][y+2] + 2*value_r[x][y+3];
		
		c5_r = 2*value_r[x+1][y-3] + 2*value_r[x+1][y-2] + 4*value_r[x+1][y-1] +
		     8*value_r[x+1][y] + 4*value_r[x+1][y+1] + 2*value_r[x+1][y+2] + 2*value_r[x+1][y+3];
		
		c6_r = value_r[x+2][y-3] + 2*value_r[x+2][y-2] + 2*value_r[x+2][y-1] +
		     4*value_r[x+2][y] + 2*value_r[x+2][y+1] + 2*value_r[x+2][y+2] + value_r[x+2][y+3];
		
		c7_r = value_r[x+3][y-3] + value_r[x+3][y-2] + 2*value_r[x+3][y-1] +
		     2*value_r[x+3][y] + 2*value_r[x+3][y+1] + value_r[x+3][y+2] + value_r[x+3][y+3];
		
		pixel_value_r[x][y] = (char)((c1_r + c2_r + c3_r + c4_r + c5_r + c6_r + c7_r) / 140);
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, pixel_value_r[x][y]);
		assval(two_dump, x, y, B, pixel_value_r[x][y]);
		assval(two_dump, x, y, G, pixel_value_r[x][y]);
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Thresholds the image currently stored in the edge data 	*/
/*structure. Any pixel with a value below 'level' is turned off.*/
/****************************************************************/
void threshold_image_proc(FL_OBJECT *n, long m)
{
short x, y, level;

level = (short)fl_get_counter_value(threshold_image);

if(level < 0 || level > 255)
{
	fl_show_message("","Only values between 0 - 255 are valid.","");
}

else
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{

			if( (screen.scr[x][y].complete_edge) >= level)
			{	
				assval(two_dump, x, y, R, screen.scr[x][y].complete_edge);
				assval(two_dump, x, y, G, screen.scr[x][y].complete_edge);
				assval(two_dump, x, y, B, screen.scr[x][y].complete_edge);
			}

			else
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			}
		}
	}
	dump_image(two_dump);
}

}

/****************************************************************/
/*Thresholds the image currently stored in the edge data 	*/
/*structure. Any pixel with a value above 'level' is turned off.*/
/****************************************************************/
void negative_threshold_image_proc(FL_OBJECT *n, long m)
{
short x, y, level;

level = (short)fl_get_counter_value(negative_threshold_image);

if(level < 0 || level > 255)
{
	fl_show_message("","Only values between 0 - 255 are valid.","");
}

else
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{

			if( (screen.scr[x][y].complete_edge) <= level)
			{	
				assval(two_dump, x, y, R, screen.scr[x][y].complete_edge);
				assval(two_dump, x, y, G, screen.scr[x][y].complete_edge);
				assval(two_dump, x, y, B, screen.scr[x][y].complete_edge);
			}

			else
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			}
		}
	}
	dump_image(two_dump);
}
}

/****************************************************************/
/*Double thresholds image by finding a high thresholded image 	*/
/*defined by 'high', and growing back missing parts of the 	*/
/*image from a lower thresholded image defined by 'low'.	*/
/****************************************************************/
void double_threshold_image_proc(FL_OBJECT *n, long m)
{
short x, y, xpos, ypos, count;
char low_array[XSIZE][YSIZE], high_array[XSIZE][YSIZE];
short pixel_original[XSIZE][YSIZE];
int low, high;
char *tmp[2];
long num_high;

tmp[0] = fl_get_input(edge_thres_low);
tmp[1] = fl_get_input(edge_thres_high);
sscanf(tmp[0], "%d", &low);
sscanf(tmp[1], "%d", &high);

for (x = 0; x < XSIZE; x++)
{
	for (y = 0; y < YSIZE; y++)
	{
		pixel_original[x][y] = valat(two_dump, x, y, R);
		low_array[x][y] = 0;
		high_array[x][y] = 0;
	}
}

for (x = 0; x < XSIZE; x++)
{
	for (y = 0; y < YSIZE; y++)
	{
		if(pixel_original[x][y] >= low)
		{
			low_array[x][y] = 255;
		}		
		
		if(pixel_original[x][y] >= high)
		{
			high_array[x][y] = 255;
		}
	}
}

do
{
num_high = 0;

for (x = 5; x < XSIZE - 5; x++)
{
	for (y = 5; y < YSIZE - 5; y++)
	{
		if(high_array[x][y] == 255)
		{
			for(xpos = x - 1; xpos <= x + 1; xpos++)
			{
				for(ypos = y - 1; ypos <= y + 1; ypos++)
				{
					if( (low_array[xpos][ypos] == 255)
					&&(high_array[xpos][ypos] == 0) )
					{
						high_array[xpos][ypos] = 255;
						low_array[xpos][ypos] = 0;
						num_high++;
					}
				}
			}
		}
	}
}

}
while(num_high > 0);

for (x = 0; x < XSIZE; x++)
{
	for (y = 0; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, high_array[x][y]);
		assval(two_dump, x, y, B, high_array[x][y]);
		assval(two_dump, x, y, G, high_array[x][y]);
		screen.scr[x][y].complete_edge = high_array[x][y];
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Saves the current display onto the file name given	   	*/
/*when prompted.                                        	*/
/****************************************************************/
void edge_save_proc(FL_OBJECT *n, long m)
{
char *fname;

fname = fl_show_file_selector("File To Save","","*.rgb","");

if(fname != NULL)
	save_rgb_image(fname,(char *)two_dump, XSIZE, YSIZE);
}

/****************************************************************/
/*Counts the number of pixels in the first 'neigh' neighbourhood*/
/*of a pixel that are at a higher value than the center pixel.  */
/*if there are more than 'level' pixels higher, then the center */
/*pixel is deleted.						*/
/****************************************************************/
void ndnms_proc(FL_OBJECT *n, long m)
{
int x, y, xpos, ypos, value;
char pixel_value[XSIZE][YSIZE], pixel_original[XSIZE][YSIZE];
char *tmp[2];
int neigh, level;

tmp[0] = fl_get_input(neigh_num_input);
tmp[1] = fl_get_input(pix_level_input);
sscanf(tmp[0], "%d", &neigh);
sscanf(tmp[1], "%d", &level);

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		pixel_value[x][y] = 0;
		pixel_original[x][y] = valat(two_dump, x, y, R);
	}
}


for(x = neigh; x < XSIZE - neigh; x++)
{
	for(y = neigh; y < YSIZE - neigh; y++)
	{
		if( (value = pixel_original[x][y]) > 0)
		{
			for(xpos = x - neigh; xpos <= x + neigh; xpos++)
			{		
				for(ypos = y - neigh; ypos <= y + neigh; ypos++)
				{
					if(pixel_original[xpos][ypos] > value)
					{
						pixel_value[x][y]++;
					}
				}
			}
		}
	}
}
		 			
for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(pixel_value[x][y] > level)
		{
			assval(two_dump, x, y, R, 0);
			assval(two_dump, x, y, B, 0);
			assval(two_dump, x, y, G, 0);
			screen.scr[x][y].complete_edge = 0;
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Erodes image by removing any pixel (that is on) that does not	*/
/*have all of its 8-neighbours on.				*/
/****************************************************************/
void erode_image_proc(FL_OBJECT *n, long m)
{
short neighbour_on;
char pixel_marked[XSIZE][YSIZE];
short x, y, xpos, ypos;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		pixel_marked[x][y] = 0;
	}
}

for(x = 1; x < XSIZE - 1; x++)
{
	for(y = 1; y < YSIZE - 1; y++)
	{
		if( (valat(two_dump, x, y, R) == 255) )
		{
			if(neigh_8(1, x, y, R, 1) != 9)
			{
				pixel_marked[x][y]++;		
			}
		
		}
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(pixel_marked[x][y] > 0)
		{
			assval(two_dump, x, y, R, 0);
			assval(two_dump, x, y, G, 0);
			assval(two_dump, x, y, B, 0);
			screen.scr[x][y].complete_edge = 0;
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Dilates image by turning on all the 8-neighbours of pixels  	*/
/*that are on. Also, to avoid squaring the image too much when 	*/
/*dilating, pixels in the 4-nieghbour directions in the second	*/
/*neighbourhood are turned on as well.				*/
/****************************************************************/
void dilate_image_proc(FL_OBJECT *n, long m)
{
short x, y, xpos, ypos;
char neighbour_off[XSIZE][YSIZE];

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		neighbour_off[x][y] = 0;
	}
}

for(x = 2; x < XSIZE - 2; x++)
{
	for(y = 2; y < YSIZE - 2; y++)
	{
		if(valat(two_dump, x, y, R) == 255)
		{
			for(xpos = x - 1; xpos <= x + 1; xpos++)
			{
				for(ypos = y - 1; ypos <= y + 1; ypos++)
				{
					if(valat(two_dump, xpos, ypos, R) == 0) 
					{
						neighbour_off[xpos][ypos]++;
					}
				}
			}
			xpos = x;
			ypos = y - 2;
	
			if(valat(two_dump, xpos, ypos, R) == 0) 
			{
				neighbour_off[xpos][ypos]++;
			}
			
			xpos = x;
			ypos = y + 2;
	
			if(valat(two_dump, xpos, ypos, R) == 0) 
			{
				neighbour_off[xpos][ypos]++;
			}
			
			xpos = x - 2;
			ypos = y;
	
			if(valat(two_dump, xpos, ypos, R) == 0) 
			{
				neighbour_off[xpos][ypos]++;
			}
			
			xpos = x + 2;
			ypos = y;
	
			if(valat(two_dump, xpos, ypos, R) == 0) 
			{
				neighbour_off[xpos][ypos]++;
			}

		}
	}
}
		
for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(neighbour_off[x][y] > 0)
		{
			assval(two_dump, x, y, R, 255);
			assval(two_dump, x, y, G, 255);
			assval(two_dump, x, y, B, 255);
			screen.scr[x][y].complete_edge = 255;
		}
	}

}
dump_image(two_dump);
}

/****************************************************************/
/*Dilates image by turing on pixels that are off that have at 	*/
/*least 2 neighbours in their first neighbourhood that are on.	*/
/****************************************************************/
void soft_dilate_proc(FL_OBJECT *n, long m)
{
short x, y;
char pixel[XSIZE][YSIZE];

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		pixel[x][y] = 0;
	}
}

for(x = 1; x < XSIZE - 1; x++)
{
	for(y = 1; y < YSIZE - 1; y++)
	{
		if(valat(two_dump, x, y, R) == 255)
		{
			pixel[x][y] = 1;
			pixel[x + 1][y] = 1;
			pixel[x - 1][y] = 1;
			pixel[x][y + 1] = 1;
			pixel[x][y - 1] = 1;
		}
	}
}			

/*
for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if( (valat(two_dump, x, y, R) == 0)
		&&(neigh_8(1, x, y, R, 1) >= 2) )
		{
			pixel[x][y] = 1;
		}

		else
		{
			pixel[x][y] = 0;
		}
	}
}		
*/

for(x = 1; x < XSIZE - 1; x++)
{
	for(y = 1; y < YSIZE - 1; y++)
	{
		if(pixel[x][y] == 1)
		{
			assval(two_dump, x, y, R, 255);
			assval(two_dump, x, y, G, 255);
			assval(two_dump, x, y, B, 255);
			screen.scr[x][y].complete_edge = 255;
		}
	}
}	

dump_image(two_dump);
}

/****************************************************************/
/*Produces a negative of the image currently being displayed.	*/
/****************************************************************/
void negative_proc(FL_OBJECT *n, long m)
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, (255 - (valat(two_dump, x, y, R))));
		assval(two_dump, x, y, G, (255 - (valat(two_dump, x, y, G))));
		assval(two_dump, x, y, B, (255 - (valat(two_dump, x, y, B))));
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Enhances edge data currently being displayed by breaking the 	*/
/*image into rectangles and re-scaling inside each rectangle.	*/
/*The re-scaling will remove the bottom 10% of pixels in the 	*/
/*rectangle and force the top 5% of pixels in the rectangle to a*/
/*grey level of 255.						*/
/****************************************************************/
void enhance_edge_pixels_proc(FL_OBJECT *n, long m)
{
short x, y, max, min, xpos, ypos;
short border_x, border_y;
char pixel_original[XSIZE][YSIZE];
short pixel_value[XSIZE][YSIZE];
long a, num_pixels;
long cut_bottom, cut_top, num_times[256], total_sum;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		pixel_original[x][y] = valat(two_dump, x, y, R);
	}
}

xpos =  79;
ypos =  60;

for(border_x = 0; border_x < XSIZE - xpos; border_x = border_x + xpos)
{
for(border_y = 0; border_y < YSIZE - ypos; border_y = border_y + ypos)
{
	total_sum = 0;
	max = -1;
	min = 256;
	num_pixels = 0;

	for(x = border_x; x < border_x + xpos; x++)
	{
		for(y = border_y; y < border_y + ypos; y++)
		{
			pixel_value[x][y] = pixel_original[x][y];
			num_pixels++;

			if(pixel_value[x][y] > max)
			{
				max = pixel_value[x][y];
			}
			
			if(pixel_value[x][y] < min)
			{
				min = pixel_value[x][y];
			}
		}
	}	
	
	for(x = 0; x <= 255; x++)
	{
		num_times[x] = 0;
	}
	
	for(x = border_x; x < border_x + xpos; x++)
	{
		for(y = border_y; y < border_y + ypos; y++)
		{
			num_times[pixel_value[x][y]]++;
		}
	}

	for(a = 0; a < max; a++)
	{
		total_sum = total_sum + num_times[a];
 		if(total_sum > num_times[0])
		{
			if(total_sum <= (long)(0.1 * (float)num_pixels))
			{
				cut_bottom = a;
			}
 		
			if(total_sum <= (long)(0.95 * (float)num_pixels))
			{
				cut_top = a;
			}
		}
		
	}
	
	if(cut_top - cut_bottom > 50) /*defensive programming*/
	{
		for(x = border_x; x < border_x + xpos; x++)
		{
			for(y = border_y; y < border_y + ypos; y++)
			{
				pixel_value[x][y] = pixel_value[x][y] - cut_bottom;
			
				if(pixel_value[x][y] < 0)
				{
					pixel_value[x][y] = 0;
				}
	
				else if(pixel_value[x][y] > (cut_top - cut_bottom))
				{
					pixel_value[x][y] = cut_top - cut_bottom;
				}
			}
		}
	
		for(x = border_x; x < border_x + xpos; x++)
		{
			for(y = border_y; y < border_y + ypos; y++)
			{
				pixel_value[x][y] = pixel_value[x][y] * 255 / (cut_top - cut_bottom);
				screen.scr[x][y].complete_edge = pixel_value[x][y];
				assval(two_dump, x, y, R, pixel_value[x][y]);
				assval(two_dump, x, y, G, pixel_value[x][y]);
				assval(two_dump, x, y, B, pixel_value[x][y]);
			}
		}
	}
}
}
dump_image(two_dump);
}

/****************************************************************/
/*Returns the number of gray-level changes around a pixel when  */
/*stepping clockwise around the pixels first neighbourhood.	*/
/****************************************************************/
short level_index(short x, short y, short colour)
{
short count, xpos, ypos, value;

count = 0;

if( ( (x > 0) && (x < XSIZE) ) && ( (y > 0) && (y < YSIZE) ) )
{
	xpos = x - 1;
	ypos = y - 1;
	value = valat(two_dump, xpos, ypos, colour);
	
	xpos++;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	xpos++;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	ypos++;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	ypos++;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	xpos--;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	xpos--;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	ypos--;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}

	ypos--;
	if(value != valat(two_dump, xpos, ypos, colour))
	{
		value = valat(two_dump, xpos, ypos, colour);
		count++;
	}
}

return(count / 2);
}

/****************************************************************/
/*Returns either the number of pixels on in the first 		*/
/*neighbourhood around a pixel, OR the value of the summed 	*/
/*chosen colour-band values around a pixel. Return is chosen by	*/
/*'send-back' and colour-band is chosen by 'colour'.		*/
/****************************************************************/
int neigh_8(short neigh_size, short x, short y, short colour, short send_back)
{
short xpos, ypos, count;
unsigned int sum;

count = 0;	
sum = 0;

if( ( (x > -1 + neigh_size) && (x < XSIZE - 1 + neigh_size) ) 
&& ( (y > -1 + neigh_size) && (y < YSIZE - 1 + neigh_size) ) )
{
	if(send_back == 0)
	{	
		for(xpos = x - neigh_size; xpos <= x + neigh_size; xpos++)
		{
			for(ypos = y - neigh_size; ypos <= y + neigh_size; ypos++)
			{
				if(valat(two_dump, xpos, ypos, colour) > 0) 
				{
					sum = sum + valat(two_dump, xpos, ypos, colour);
				}
			}
		}
		
		return(sum);
	}
	
	else /*if(send_back == 1)*/
	{	
		for(xpos = x - neigh_size; xpos <= x + neigh_size; xpos++)
		{
			for(ypos = y - neigh_size; ypos <= y + neigh_size; ypos++)
			{
				if(valat(two_dump, xpos, ypos, colour) > 0) 
				{
					count++;
				}
			}
		}
		
		return(count);
	}
}

else
{
	return(-1);
}

} 

/****************************************************************/
/*Thins the current image to 1-pixel-wide boundaries without 	*/
/*breaking boudaries or removing open ended line segments.	*/
/****************************************************************/
void skeleton_image_proc(FL_OBJECT *n, long m)
{
short x, y; 
short temp[XSIZE][YSIZE], pixel[XSIZE][YSIZE];
unsigned long num_pix;

for(y = 0; y < YSIZE; y++)
{
	for(x = 0; x < XSIZE; x++)
	{
		temp[x][y] = 0;
		pixel[x][y] = valat(two_dump, x, y, R);
	}
}

do
{
num_pix = 0;

for(x = 6; x < XSIZE - 6; x++)
{
	for(y = 6; y < YSIZE - 6; y++)
	{
		if(pixel[x][y] > 0)
		{
			if( (neigh_8(1, x, y, R, 1) >= 4) && (neigh_8(1, x, y, R, 1) <= 6)
			&&(temp[x][y - 1] == 0)
			&&(temp[x - 1][y] == 0) )
			{
				if(level_index(x, y, R) == 1)
				{
					temp[x][y] = 1;						
				}
			}
		}
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(temp[x][y] == 1)
		{
			assval(two_dump, x, y, R, 0);
			assval(two_dump, x, y, G, 0);
			assval(two_dump, x, y, B, 0);
			screen.scr[x][y].complete_edge = 0;
			num_pix++;
			pixel[x][y] = 0;
			temp[x][y] = 0;
		}
	}
}
dump_image(two_dump);
}
while(num_pix > 0);
}

/****************************************************************/
/*Overlays the edge data currently in the edge data structure 	*/
/*overtop of the chosen image. Chosen image can be one of max, 	*/
/*min, average, max_pos or min_pos.				*/
/****************************************************************/
void overlay_image_proc(FL_OBJECT *n, long m)
{
short x, y; 

if(zoom_flag == 1)
	zoom_flag = 0;
	
if(current_image_flag == E_EDGE)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			if(screen.scr[x][y].complete_edge > 0)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 255);
			}

			else
			{
				assval(two_dump, x, y, R, screen.scr[x][y].max[SR]);
				assval(two_dump, x, y, G, screen.scr[x][y].max[SG]);
				assval(two_dump, x, y, B, screen.scr[x][y].max[SB]);
			}			
		}
	}
	current_image_flag = E_MAX;
}

else
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			if(screen.scr[x][y].complete_edge > 0)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 255);
			}
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Removes edge data from overtop a chosen image.		*/
/****************************************************************/
void un_overlay_image_proc(FL_OBJECT *n, long m)
{
short x, y; 
char pixel_high[XSIZE][YSIZE];
char pix_r[XSIZE][YSIZE];
char pix_g[XSIZE][YSIZE];
char pix_b[XSIZE][YSIZE];

if(zoom_flag == 1)
	zoom_flag = 0;

if(current_image_flag == E_AVERAGE)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].tot[SR] / screen.no_frames;
			pix_g[x][y] = screen.scr[x][y].tot[SG] / screen.no_frames;
			pix_b[x][y] = screen.scr[x][y].tot[SB] / screen.no_frames;
		}
	}
}

else if(current_image_flag == E_MAX)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].max[SR];
			pix_g[x][y] = screen.scr[x][y].max[SG];
			pix_b[x][y] = screen.scr[x][y].max[SB];
		}
	}
}

else if(current_image_flag == E_MIN)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].min[SR];
			pix_g[x][y] = screen.scr[x][y].min[SG];
			pix_b[x][y] = screen.scr[x][y].min[SB];
		}
	}
}

else if(current_image_flag == E_MAX_POS)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].max_pos[SR];
			pix_g[x][y] = screen.scr[x][y].max_pos[SG];
			pix_b[x][y] = screen.scr[x][y].max_pos[SB];
		}
	}
}

else if(current_image_flag == E_MIN_POS)
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].min_pos[SR];
			pix_g[x][y] = screen.scr[x][y].min_pos[SG];
			pix_b[x][y] = screen.scr[x][y].min_pos[SB];
		}
	}
}

else
{
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			pix_r[x][y] = screen.scr[x][y].complete_edge;
			pix_g[x][y] = screen.scr[x][y].complete_edge;
			pix_b[x][y] = screen.scr[x][y].complete_edge;
		}
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(screen.scr[x][y].complete_edge > 0)
		{
			pixel_high[x][y] = 1;
		}

		else
		{
			pixel_high[x][y] = 0;
		}
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(pixel_high[x][y] == 1)
		{
			assval(two_dump, x, y, R, pix_r[x][y]);
			assval(two_dump, x, y, G, pix_g[x][y]);
			assval(two_dump, x, y, B, pix_b[x][y]);
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Pulls all pixels that are on to a level of 255 for all three	*/
/*channels.							*/
/****************************************************************/
void pull_up_proc(FL_OBJECT *n, long m)
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if( (valat(two_dump, x, y, R) > 0)
		||(valat(two_dump, x, y, G) > 0)
		||(valat(two_dump, x, y, B) > 0) )
		{
			assval(two_dump, x, y, R, 255);
			assval(two_dump, x, y, B, 255);
			assval(two_dump, x, y, G, 255);
			screen.scr[x][y].complete_edge = 255;
		}
	
		else
		{
			screen.scr[x][y].complete_edge = 0;
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Restores the initial edge data to the screen and to the edge	*/
/*data structure.						*/
/****************************************************************/
void undo_all_proc(FL_OBJECT *n, long m)
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, screen.scr[x][y].sample_edge);
		assval(two_dump, x, y, B, screen.scr[x][y].sample_edge);
		assval(two_dump, x, y, G, screen.scr[x][y].sample_edge);
		screen.scr[x][y].complete_edge = screen.scr[x][y].sample_edge;
	}
}
dump_image(two_dump);

current_image_flag = E_EDGE;

if( (zoom_flag == 1)
||(edit_flag == 1) )
{
	edit_flag = 0;
	zoom_flag = 0;
	fl_set_event_call_back(dummy);
}
}

/****************************************************************/
/*Gets one of max, min, edge, average, max_pos or min_pos 	*/
/*and dumps it into the edge window.				*/
/****************************************************************/
void edge_get_image_proc(FL_OBJECT *n, long m)
{
short x, y;

if(m == 0)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].tot[SR] / screen.no_frames);
			assval(two_dump, x, y, B, screen.scr[x][y].tot[SB] / screen.no_frames);
			assval(two_dump, x, y, G, screen.scr[x][y].tot[SG] / screen.no_frames);
		}
	}
	current_image_flag = E_AVERAGE;
}

else if(m == 1)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].max[SR]);
			assval(two_dump, x, y, B, screen.scr[x][y].max[SB]);
			assval(two_dump, x, y, G, screen.scr[x][y].max[SG]);
		}
	}
	current_image_flag = E_MAX;
}

else if(m == 2)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].min[SR]);
			assval(two_dump, x, y, B, screen.scr[x][y].min[SB]);
			assval(two_dump, x, y, G, screen.scr[x][y].min[SG]);
		}
	}
	current_image_flag = E_MIN;
}

else if(m == 3)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].max_pos[SR]);
			assval(two_dump, x, y, B, screen.scr[x][y].max_pos[SB]);
			assval(two_dump, x, y, G, screen.scr[x][y].max_pos[SG]);
		}
	}
	current_image_flag = E_MAX_POS;
}

else if(m == 4)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].min_pos[SR]);
			assval(two_dump, x, y, B, screen.scr[x][y].min_pos[SB]);
			assval(two_dump, x, y, G, screen.scr[x][y].min_pos[SG]);
		}
	}
	current_image_flag = E_MIN_POS;
}

else
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, screen.scr[x][y].complete_edge);
			assval(two_dump, x, y, B, screen.scr[x][y].complete_edge);
			assval(two_dump, x, y, G, screen.scr[x][y].complete_edge);
		}
	}
	current_image_flag = E_EDGE;
}
zoom_flag = 0;
dump_image(two_dump);
}

/****************************************************************/
/*Toggles the R, G and B channels on or off for max, average  	*/
/*and min images.						*/
/****************************************************************/
void edge_colours_proc(FL_OBJECT *n, long m)
{
short x, y, needed = 0;
int r_on, g_on, b_on;
static char memory_r[XSIZE][YSIZE];
static char memory_g[XSIZE][YSIZE];
static char memory_b[XSIZE][YSIZE];

if(current_image_flag == E_AVERAGE)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			memory_r[x][y] = screen.scr[x][y].tot[SR]/screen.no_frames;
			memory_g[x][y] = screen.scr[x][y].tot[SG]/screen.no_frames;
			memory_b[x][y] = screen.scr[x][y].tot[SB]/screen.no_frames;
		}
	}
	needed = 1;
}

else if(current_image_flag == E_MAX)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			memory_r[x][y] = screen.scr[x][y].max[SR];
			memory_g[x][y] = screen.scr[x][y].max[SG];
			memory_b[x][y] = screen.scr[x][y].max[SB];
		}
	}
	needed = 1;
}

else if(current_image_flag == E_MIN)
{
	for (x = 0; x < XSIZE; x++)
	{
		for (y = 0; y < YSIZE; y++)
		{
			memory_r[x][y] = screen.scr[x][y].min[SR];
			memory_g[x][y] = screen.scr[x][y].min[SG];
			memory_b[x][y] = screen.scr[x][y].min[SB];
		}
	}
	needed = 1;
}

if(needed)
{
	r_on = fl_get_button(edge_r);
	g_on = fl_get_button(edge_g);
	b_on = fl_get_button(edge_b);
	
	if( (r_on == 0) && (g_on == 0) && (b_on == 0) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, memory_r[x][y]);
				assval(two_dump, x, y, G, memory_g[x][y]);
				assval(two_dump, x, y, B, memory_b[x][y]);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 1) && (g_on == 1) && (b_on == 1) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 0) && (g_on == 1) && (b_on == 1) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, memory_r[x][y]);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 0) && (g_on == 0) && (b_on == 1) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, memory_r[x][y]);
				assval(two_dump, x, y, G, memory_g[x][y]);
				assval(two_dump, x, y, B, 0);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 0) && (g_on == 1) && (b_on == 0) ) 
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, memory_r[x][y]);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, memory_b[x][y]);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 1) && (g_on == 1) && (b_on == 0) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, memory_b[x][y]);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 1) && (g_on == 0) && (b_on == 1) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, memory_g[x][y]);
				assval(two_dump, x, y, B, 0);
			}
		}
		dump_image(two_dump);
	}

	else if( (r_on == 1) && (g_on == 0) && (b_on == 0) )
	{
		for (x = 0; x < XSIZE; x++)
		{
			for (y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, memory_g[x][y]);
				assval(two_dump, x, y, B, memory_b[x][y]);
			}
		}
		dump_image(two_dump);
	}
}
}

/****************************************************************/
/*fills in matrix for quartzite and sandstone edges.		*/
/****************************************************************/
void fill_matrix_proc(FL_OBJECT *n, long m)
{
short x, y;
long num_pix;

edge_get_image_proc(NULL, E_EDGE); 
border_image_proc(NULL, 0);

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		protected[x][y] = 0;
	}
}

matrix_fill_image();

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(protected[x][y] == 0)
		{
			assval(two_dump, x, y, R, 255);
			assval(two_dump, x, y, G, 255);
			assval(two_dump, x, y, B, 255);
			screen.scr[x][y].complete_edge = 255;
		}
	}
}
							
for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(screen.scr[x][y].complete_edge == 0)
		{
			assval(two_dump, x, y, R, 0);
			assval(two_dump, x, y, G, 0);
			assval(two_dump, x, y, B, 0);
		}
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Cleans image by first calling fill and then clean.		*/
/****************************************************************/
void clean_image_proc(FL_OBJECT *n, long m)
{
short x, y;

edge_get_image_proc(NULL, E_EDGE); 
border_image_proc(NULL, 0);
boundary_fill_image_proc(NULL, 0);

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(screen.scr[x][y].complete_edge == 0)
		{
			assval(two_dump, x, y, R, 0);
			assval(two_dump, x, y, G, 0);
			assval(two_dump, x, y, B, 0);
		}
	}
}
dump_image(two_dump);

clean_image();
clean_image();

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		screen.scr[x][y].complete_edge = valat(two_dump, x, y, R);
	}
}
}

/****************************************************************/
/*Cleans image using the fill routine in module measure.c.	*/
/*Checks to see that pixel is not surrounded by the same grain. */
/*If it is it is deleted, if it is not is left alone.		*/
/*Requires fill routine to be run before calling this routine.	*/
/****************************************************************/
void clean_image()
{
short x, y;
int rad;

for(x = 6; x < XSIZE - 6; x++)
{
	for(y = 6; y < YSIZE - 6; y++)
	{
		rad = 6;
		do{
		
		if( (screen.types[x][y].grain_number == 0)/*edge pixel*/
		&&(screen.types[x][y - 1].grain_number > 0)/*not edge pixel*/
		&&(screen.types[x][y + rad].grain_number > 0) )
		{
			if(screen.types[x][y - 1].grain_number == screen.types[x][y + rad].grain_number)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			
				screen.types[x][y].grain_number = screen.types[x][y + rad].grain_number;
			}
		}

		if( (screen.types[x][y].grain_number == 0)
		&&(screen.types[x - 1][y].grain_number > 0)
		&&(screen.types[x + rad][y].grain_number > 0) )
		{
			if(screen.types[x - 1][y].grain_number == screen.types[x + rad][y].grain_number)
			{
				assval(two_dump, x, y, R, 0);
				assval(two_dump, x, y, G, 0);
				assval(two_dump, x, y, B, 0);
			
				screen.types[x][y].grain_number = screen.types[x + rad][y].grain_number;
			}
		}
		rad = rad - 1;
		}
		while(rad > 0);
	}
}
dump_image(two_dump);			
}

/****************************************************************/
/*Puts a border around the current image in the edge window.	*/
/*This is necessary before filling is attempted or you WILL GET */
/*A CORE DUMP.							*/
/****************************************************************/
void border_image_proc(FL_OBJECT *n, long m)
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y <= 5; y++)
	{
		assval(two_dump, x, y, R, 255);
		assval(two_dump, x, y, G, 255);
		assval(two_dump, x, y, B, 255);
		screen.scr[x][y].complete_edge = 255;
	}
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 480; y < YSIZE; y++)
	{
		assval(two_dump, x, y, R, 255);
		assval(two_dump, x, y, G, 255);
		assval(two_dump, x, y, B, 255);
		screen.scr[x][y].complete_edge = 255;
	}
}

for(y = 0; y < YSIZE; y++)
{
	for(x = 0; x <= 5; x++)
	{
		assval(two_dump, x, y, R, 255);
		assval(two_dump, x, y, G, 255);
		assval(two_dump, x, y, B, 255);
		screen.scr[x][y].complete_edge = 255;
	}
}

for(y = 0; y < YSIZE; y++)
{
	for(x = 634; x < XSIZE; x++)
	{
		assval(two_dump, x, y, R, 255);
		assval(two_dump, x, y, G, 255);
		assval(two_dump, x, y, B, 255);
		screen.scr[x][y].complete_edge = 255;
	}
}
dump_image(two_dump);
}

/****************************************************************/
/*Main procedure to pull edges out of the gradient image (and 	*/
/*altered gradient image in procedure4). Self explainitory.	*/
/****************************************************************/
void auto_get_edges_proc(FL_OBJECT *n, long m)
{
gaussian_filter_image_proc(NULL, 0);
enhance_edge_pixels_proc(NULL, 0);
gaussian_filter_image_proc(NULL, 0);
enhance_edge_pixels_proc(NULL, 0);
ndnms_proc(NULL, 0);
double_threshold_image_proc(NULL, 0);
soft_dilate_proc(NULL, 0);
skeleton_image_proc(NULL, 0);
soft_dilate_proc(NULL, 0);
clean_image_proc(NULL, 0);
}

/****************************************************************/
/*Refreshes edge window.					*/
/****************************************************************/
void edge_refresh_proc(FL_OBJECT *n, long m)
{
dump_image(two_dump);
}

/****************************************************************/
/*Closes edit mode on mouse click events.			*/
/****************************************************************/
void stop_edit_edge_proc(FL_OBJECT *n, long m)
{
if(edit_flag == 1)
{
	fl_set_event_call_back(dummy);
	edit_flag = 0;
}
}

/****************************************************************/
/*The callback for the edit mouse event is set up here.		*/
/****************************************************************/
void edit_edge_proc(FL_OBJECT *n, long m)
{
fl_set_event_call_back(edit_callback);
if(edit_flag == 0)
	edit_flag = 1;
}

/****************************************************************/
/*Dummy callback routine for when mouse event is not wanted.	*/
/****************************************************************/
void dummy(short dev, short val)
{
}

/****************************************************************/
/*Procedure is called when the left or right mouse button is	*/
/*pushed inside the (active) edge window. Captures x-y position	*/
/*of mouse in window.						*/
/****************************************************************/
void edit_callback(short dev, short val)
{
short x, y, choice, zoom; 
long xx, yy;
	
if(fl_get_button(edit_remove) == 1)
{
	choice = 0;
}

else
{
	choice = 255;
}

if(zoom_flag == 1)
{
	zoom = 1;
}

else 
{
	zoom = 0;
}

getorigin(&xx, &yy);
while(getbutton(LEFTMOUSE))
{
	x = (short)(getvaluator(MOUSEX) - xx);
	y = YSIZE - (short)(getvaluator(MOUSEY) - yy);
	
	if(x >= 1 && x < XSIZE - 1 && y >= 1 && y < YSIZE - 1)
		edit_edge(x, y, choice, zoom);
}
}

/****************************************************************/
/*Removes a 1-neighbourhood wide square on choice '0'.		*/
/*Adds a 1-neighbourhood wide square on choice '255'.		*/
/****************************************************************/
void edit_edge(short x, short y, char choice, short zoom)
{
short xpos, ypos, val;
long cmd;

if(zoom == 0)
{
	for(xpos = x - 1; xpos <= x + 1; xpos++)
	{
		for(ypos = y - 1; ypos <= y + 1; ypos++)
		{
			assval(two_dump, xpos, ypos, R, choice);
			assval(two_dump, xpos, ypos, G, choice);
			assval(two_dump, xpos, ypos, B, choice);
			screen.scr[xpos][ypos].complete_edge = choice;
		}
	}
	dump_image(two_dump);
}

else
{
	for(xpos = x - 1; xpos <= x + 1; xpos++)
	{
		for(ypos = y - 1; ypos <= y + 1; ypos++)
		{
			assval((char *)zoom_buffer, xpos, ypos, R, choice);
			assval((char *)zoom_buffer, xpos, ypos, G, choice);
			assval((char *)zoom_buffer, xpos, ypos, B, choice);
			screen.scr[(xpos / 2) + xzoom_zero][(ypos / 2) + yzoom_zero]
				.complete_edge = choice;
		}
	}
	dump_image((char *)zoom_buffer);
}

}

/****************************************************************/
/*The callback for the zoom mouse event is set up here.		*/
/****************************************************************/
void zoom_edge_proc(FL_OBJECT *n, long m)
{
if( (edit_flag == 1)
&&(zoom_flag == 0) )
{
	edit_flag = 0;
}

else if( (edit_flag == 0)
&&(zoom_flag == 1) )
{
	edge_get_image_proc(NULL, current_image_flag); 
}

else if( (zoom_flag == 1)
&&(edit_flag == 1) )
{
	edit_flag = 0;
	edge_get_image_proc(NULL, current_image_flag);
}

fl_set_event_call_back(zoom_callback);
zoom_flag = 0;
}

/****************************************************************/
/*Zooms in on selected square area.				*/
/****************************************************************/
void zoom_callback(short dev, short val)
{
long xx, yy;
short xc, yc;

getorigin(&xx, &yy);

xc = (short)(getvaluator(MOUSEX) - xx);
yc = YSIZE - (short)(getvaluator(MOUSEY) - yy);

xzoom_zero = xc - XSIZE / 4;
yzoom_zero = yc - YSIZE / 4;

if(xc < XSIZE / 4)
	xzoom_zero = 0; 
	
if(xc > XSIZE - XSIZE / 4)
	xzoom_zero = (XSIZE / 2) - 1; 

if(yc < YSIZE / 4)
	yzoom_zero = 0; 

if(yc > YSIZE - YSIZE / 4)
	yzoom_zero = (YSIZE / 2) - 1; 

show_zoom_image();

fl_set_event_call_back(dummy);
zoom_flag = 1;
}

/****************************************************************/
/*Displays a 2x zoomed-in image around selected pixel.		*/
/****************************************************************/
void show_zoom_image()
{
short x, y, xpos, ypos;
char temp_buf[XSIZE][YSIZE][3];

for(x = 0; x < XSIZE / 2; x++)
{
	for(y = 0; y < YSIZE / 2; y++)
	{
		assval((char *)temp_buf, x, y, R, valat(two_dump, xzoom_zero + x, 
								  yzoom_zero + y, R));
		assval((char *)temp_buf, x, y, G, valat(two_dump, xzoom_zero + x, 
								  yzoom_zero + y, G));
		assval((char *)temp_buf, x, y, B, valat(two_dump, xzoom_zero + x, 
								  yzoom_zero + y, B));
	}
}

for(x = 0; x < XSIZE / 2; x++)
{
	for(y = 0; y < YSIZE / 2; y++)
	{
		xpos = x * 2;
		ypos = y * 2;
		assval((char *)zoom_buffer, xpos, ypos, R, valat((char *)temp_buf, x, y, R));
		assval((char *)zoom_buffer, xpos, ypos, G, valat((char *)temp_buf, x, y, G));
		assval((char *)zoom_buffer, xpos, ypos, B, valat((char *)temp_buf, x, y, B));

		xpos = (x * 2) + 1;
		ypos = y * 2;
		assval((char *)zoom_buffer, xpos, ypos, R, valat((char *)temp_buf, x, y, R));
		assval((char *)zoom_buffer, xpos, ypos, G, valat((char *)temp_buf, x, y, G));
		assval((char *)zoom_buffer, xpos, ypos, B, valat((char *)temp_buf, x, y, B));

		xpos = x * 2;
		ypos = (y * 2) + 1;
		assval((char *)zoom_buffer, xpos, ypos, R, valat((char *)temp_buf, x, y, R));
		assval((char *)zoom_buffer, xpos, ypos, G, valat((char *)temp_buf, x, y, G));
		assval((char *)zoom_buffer, xpos, ypos, B, valat((char *)temp_buf, x, y, B));
		
		xpos = (x * 2) + 1;
		ypos = (y * 2) + 1;
		assval((char *)zoom_buffer, xpos, ypos, R, valat((char *)temp_buf, x, y, R));
		assval((char *)zoom_buffer, xpos, ypos, G, valat((char *)temp_buf, x, y, G));
		assval((char *)zoom_buffer, xpos, ypos, B, valat((char *)temp_buf, x, y, B));

	}
}					
dump_image((char *)zoom_buffer);
}

/****************************************************************/
/*Closes edge window and edge menu and returns to the main menu.*/
/****************************************************************/
void done_proc(FL_OBJECT *n, long m)
{
winclose(edge_win);
fl_hide_object(edge_grp);
edge_win = -1;
fl_show_object(main_grp);
edit_flag = 0;
zoom_flag = 0;
fl_unqdevice(LEFTMOUSE);
fl_set_event_call_back(dummy);
}


