#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include <math.h>
#include "forms.h"
#include "gv_forms.h"
#include "types.h"
#include "hist.h"

extern screenT screen;
extern char zoom_buffer[XSIZE][YSIZE][3];
extern char dump[XSIZE][YSIZE][4];
extern char two_dump[XSIZE][YSIZE][3];
extern short zoom_flag;
extern long edge_win, i_win;
long histogram_win = -1, current_win = -1;

static char line_val_r[XSIZE][YSIZE];
static char line_val_g[XSIZE][YSIZE];
static char line_val_b[XSIZE][YSIZE];

static char image_mem_r[XSIZE][YSIZE];
static char image_mem_g[XSIZE][YSIZE];
static char image_mem_b[XSIZE][YSIZE];

static long rc[256];
static long gc[256];
static long bc[256];
static long maxr, maxg, maxb;
static long minr, ming, minb;

static short xs, ys, xe, ye;

static char line_flag[XSIZE][YSIZE];

/**************************************************************/
/*Opens histogram menu and line histogram window. Also,       */
/*plots histogarm of entire image.			      */	
/**************************************************************/
void info_hist_proc(FL_OBJECT *n, long m)
{
short x, y;

fl_hide_all();
fl_show_object(hist_grp);

if(m == 0)
{
	current_win = i_win;
	
	for(x = 0; x < XSIZE; x++)
	{
		for(y = 0; y < YSIZE; y++)
		{
			assval(two_dump, x, y, R, valat(dump, x, y, R));
			assval(two_dump, x, y, G, valat(dump, x, y, G));
			assval(two_dump, x, y, B, valat(dump, x, y, B));
		}
	}
}

else
{
	current_win = edge_win;
	if(zoom_flag == 1)
	{
		for(x = 0; x < XSIZE; x++)
		{
			for(y = 0; y < YSIZE; y++)
			{
				assval(two_dump, x, y, R, valat(zoom_buffer, x, y, R));
				assval(two_dump, x, y, G, valat(zoom_buffer, x, y, G));
				assval(two_dump, x, y, B, valat(zoom_buffer, x, y, B));
			}
		}
	}
}

prefsize(XSIZE, 260);
histogram_win = winopen("LINE HISTOGRAM");
gconfig();
color(WHITE);
clear();
init_hist_callback();
hist_entire_image_proc(NULL, 0);
}

/**************************************************************/
/*Refreshes window when hist menu is open.		      */	
/**************************************************************/
void hist_refresh_proc(FL_OBJECT *n, long m)
{
winset(current_win);
dump_image(two_dump);
}

/**************************************************************/
/*Removes old histogram line from the image in preparation    */
/*for plotting a new histogram.				      */
/**************************************************************/
void remove_hist_line()
{
register short x, y;

for (x = 0; x < XSIZE; x++)
{
	for (y = 0; y < YSIZE; y++)
	{
		if(line_flag[x][y] == 255)
		{
			assval(two_dump, x, y, R, line_val_r[x][y]);
			assval(two_dump, x, y, G, line_val_g[x][y]);
			assval(two_dump, x, y, B, line_val_b[x][y]);
			line_flag[x][y] = 0;
		}
	}
}
}

/**************************************************************/
/*Dummy procedure set up as a callback for the mouse click    */
/*event. Takes care of the case when we do not want the click */
/*event to do anything.					      */
/**************************************************************/
void hist_dummy_callback(short dev, short val)
{
}

/**************************************************************/
/*The callback procedure for the mouse event is set up here.  */
/*The setup for the area histogram is also done here.         */
/**************************************************************/
void init_hist_callback()
{
short x, y;

/*set up for area histogram*/

for(x = 0; x <= 255; x++)
{
	rc[x] = 0;
	gc[x] = 0;
	bc[x] = 0;
}

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		rc[valat(two_dump, x, y, R)]++;
		gc[valat(two_dump, x, y, G)]++;
		bc[valat(two_dump, x, y, B)]++;
	}
}

maxr = 0;
minr = 10000;
maxg = 0;
ming = 10000;
maxb = 0;
minb = 10000;

for(x = 0; x <= 255; x++)
{
	if(rc[x] > maxr)
		maxr = rc[x];

	if(gc[x] > maxg)
		maxg = gc[x];

	if(bc[x] > maxb)
		maxb = bc[x];

	if(rc[x] < minr)
		minr = rc[x];

	if(gc[x] < ming)
		ming = gc[x];

	if(bc[x] < minb)
		minb = bc[x];
}

/*callback for the mouse event*/

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		line_flag[x][y] = 0;
		image_mem_r[x][y] = valat(two_dump, x, y, R);
		image_mem_g[x][y] = valat(two_dump, x, y, G);
		image_mem_b[x][y] = valat(two_dump, x, y, B);
	}
}

fl_set_event_call_back(hist_callback);
winset(current_win);
}

/**************************************************************/
/*Procedure is called when the left mouse button is pushed    */
/*inside the (active) image window. Captures x-position and   */
/*y-position of mouse in window.			      */
/**************************************************************/
void hist_callback(short dev, short val)
{
long xx, yy;
int length;
char r[XSIZE], g[XSIZE], b[XSIZE];

getorigin(&xx, &yy);
xs = (short)(getvaluator(MOUSEX) - xx);
ys = YSIZE - (short)(getvaluator(MOUSEY) - yy);

while(getbutton(LEFTMOUSE))
{
	xe = (short)(getvaluator(MOUSEX) - xx);
	ye = YSIZE - (short)(getvaluator(MOUSEY) - yy);

	if( ((xe < XSIZE) && (xe >= 0))
	&&((ye < YSIZE) && (ye >= 0)) )
	{
		remove_hist_line();
		length = make_buffers(r, g, b);
		make_line_histogram(r, g, b, length);
	}
		
	else
	{
		while(getbutton(LEFTMOUSE));
	}
}
winset(current_win);
}
 	
/**************************************************************/
/*Produces a histogram of the entire image.		      */
/**************************************************************/
void hist_entire_image_proc(FL_OBJECT *n, long m)
{
short x, y;
int col_choice;

col_choice = fl_get_button(hist_def_col);
fl_clear_chart(area_hist_plot);
fl_freeze_form(main_form);
if(col_choice == 1)
{
	fl_set_chart_bounds(area_hist_plot, (float)minr, (float)maxr);
	
	for(x = 0; x <= 255; x+=3)
	{
		fl_add_chart_value(area_hist_plot,
				  (float)((rc[x] + gc[x] + bc[x]) / 3), "", WHITE);
	}
}

else if(col_choice == 2)
{
	fl_set_chart_bounds(area_hist_plot, (float)minr, (float)maxr);
	for(x = 0; x <= 255; x+=3)
	{
		fl_add_chart_value(area_hist_plot, (float)rc[x], "", RED);
	}
}

else if(col_choice == 3)
{
	fl_set_chart_bounds(area_hist_plot, (float)ming, (float)maxg);
	for(x = 0; x <= 255; x+=3)
	{
		fl_add_chart_value(area_hist_plot, (float)gc[x], "", GREEN);
	}
}

else
{
	fl_set_chart_bounds(area_hist_plot, (float)minb, (float)maxb);
	for(x = 0; x <= 255; x+=3)
	{
		fl_add_chart_value(area_hist_plot, (float)bc[x], "", BLUE);
	}
}
fl_unfreeze_form(main_form);
}

/*************************************************************/
/*Fills and returns the length of the buffers.		     */	
/*************************************************************/
int make_buffers(char *r, char *g, char *b)
{
register short x, y;
float slope;
double angle, pi = 2.0 * acos(0.0);
short xstart, ystart;
short xend, yend;
short temp, counter;

for(x = 0; x < XSIZE; x++)
{
	r[x] = 0;
	b[x] = 0;
	g[x] = 0;
}
	
angle = atan2( (ye - ys), (xe - xs) );

if( ((angle <= (pi / 4.0)) && (angle >= -pi / 4.0))
||((angle >= (3.0 * pi / 4.0)) && (angle <= pi))
||((angle <= (-3.0 * pi / 4.0)) && (angle >= -pi)) )
{
	if(xs < xe)
	{
		xstart = xs;
		xend = xe;
	}

	else
	{
		xstart = xe;
		xend = xs;
	}

	slope = ((float)(ye - ys)) / ((float)(xe - xs));

	counter = 0;
	for (x = xstart; x <= xend; x++)
	{
		y = ys + (short)(slope * ((float)(x - xs)));
		
		r[counter] = image_mem_r[x][y];
		g[counter] = image_mem_g[x][y];
		b[counter] = image_mem_b[x][y];

		line_val_r[x][y] = r[counter];
		line_val_b[x][y] = b[counter];
		line_val_g[x][y] = g[counter];
		line_flag[x][y] = 255;

		assval(two_dump, x, y, R, 0);
		assval(two_dump, x, y, G, 0);
		assval(two_dump, x, y, B, 255);

		counter++;
	}
	
	dump_image(two_dump);
	return(xend - xstart);
}

else 
{
	if(ys < ye)
	{
		ystart = ys;
		yend = ye;
	}

	else
	{
		ystart = ye;
		yend = ys;
	}

	slope = ((float)(xe - xs)) / ((float)(ye - ys));

	counter = 0;
	for (y = ystart; y <= yend; y++)
	{
		x = xs + (short)(slope * ((float)(y - ys)));
		
		r[counter] = image_mem_r[x][y];
		g[counter] = image_mem_g[x][y];
		b[counter] = image_mem_b[x][y];

		line_val_r[x][y] = r[counter];
		line_val_b[x][y] = b[counter];
		line_val_g[x][y] = g[counter];
		line_flag[x][y] = 255;

		assval(two_dump, x, y, R, 0);
		assval(two_dump, x, y, G, 0);
		assval(two_dump, x, y, B, 255);

		counter++;
	}
	
	dump_image(two_dump);
	return(yend - ystart);
}
}

/*************************************************************/
/*Draws a user defined histogram in the appropriate window.  */
/*************************************************************/
void make_line_histogram(char *r, char *g, char *b, int length)
{
long vector[2],i;
int ch_col;

ch_col = fl_get_choice(hist_def_col);
winset(histogram_win);
clear();
	
if(ch_col == 1)
{
	bgnline();
	color(RED);
		
	for (i = 0; i < length; i++)
	{
		vector[0] = i;
		vector[1] = (r[i] +g[i] +b[i]) /3;
		v2i(vector);
	}
	endline();
}

else if(ch_col == 2)
{
	bgnline();
	color(RED);
		
	for (i = 0; i < length; i++)
	{
		vector[0] = i;
		vector[1] = r[i];
		v2i(vector);
	}
	endline();
}

else if(ch_col == 3)
{
	bgnline();
	color(GREEN);
	for (i = 0; i < length; i++)
	{
		vector[0] = i;
		vector[1] = g[i];
		v2i(vector);
	}
	endline();
}

/*
if(ch_col == 1 || ch_col == 4)
*/
else
{
	bgnline();
	color(BLUE);
	for (i = 0; i < length; i++)
	{
		vector[0] = i;
		vector[1] = b[i];
		v2i(vector);
	}
	endline();
}
gflush();
color(WHITE);
winset(current_win);
}

/**************************************************************/
/*Closes the histogram window and menu.			      */
/**************************************************************/
void hist_done_proc(FL_OBJECT *n, long m)
{
winclose(histogram_win);
winset(current_win);
remove_hist_line();
dump_image(two_dump);

fl_hide_object(hist_grp);

if(current_win == edge_win)
{	
	fl_show_object(edge_grp);
	fl_set_event_call_back(hist_dummy_callback);
}

else
{
	fl_show_object(info_grp);
	fl_set_event_call_back(info_callback);
}

histogram_win = -1;
current_win = -1;
}


