/********************************************************/
/*	INFO MODULE					*/
/*	This moule is charge of displaying the sample	*/
/* 	in several number of domains.			*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/

#include <gl/gl.h>
#include <gl/device.h>
#include <stdio.h>
#include <stdlib.h>
#include <dmedia/vl.h>
#include "forms.h"
#include "gv_forms.h"
#include "types.h"
#include "info.h"

extern screenT screen;
static curr_but = 0;

/* SHOULD BE ALLOCATED ON THE FLY */
char dump[XSIZE][YSIZE][4];

long i_win = -1;

/********************************************************/
/* Dummy procedure set up as a callback for the mouse	*/
/* click event. It is to take care of the case when	*/
/* are out of this module and we dont want forms to 	*/
/* invoke the real procedure on a mouse click		*/
/********************************************************/
void nothing(short dev, short val)
{
}

/********************************************************/
/* Returns us to the main menu				*/
/********************************************************/
void i_done_proc(FL_OBJECT *n, long m)
{
winclose(i_win);
fl_unqdevice(LEFTMOUSE);
fl_hide_all();
fl_show_object(main_grp);
fl_set_event_call_back(nothing);
}

/********************************************************/
/* This procedure is invoked by the buttons on the	*/
/* menu. The domain is determined by the variable m	*/
/* which is defined differently for each on of the 	*/
/* buttons. It also sets the current domain as the 	*/
/* global variable					*/
/********************************************************/
void info_disp_proc(FL_OBJECT *n, long m)
{
curr_but = m;
draw_data((char)m, FALSE);
}

/********************************************************/
/* Displays the sample information in the current 	*/
/* domain.						*/
/********************************************************/
void info_display_curr(FL_OBJECT *n, long m)
{
draw_data(curr_but, FALSE);
}

/********************************************************/
/* Minimum and maximum for all bands is established	*/
/* and the values are stretched accordingly to range	*/
/* between 0 and 255					*/
/********************************************************/
void info_stretch_proc(FL_OBJECT *n, long m)
{
int min = 200, max = 0;
int diff, temp,i,j;

for (i=0; i<XSIZE; i++)
for (j=0; j<YSIZE; j++)
{
	if (dump[i][j][R] > max)
		max = dump[i][j][R];
	if (dump[i][j][G] > max)
		max = dump[i][j][G];
	if (dump[i][j][B] > max)
		max = dump[i][j][B];
	if (dump[i][j][R] < min)
		min = dump[i][j][R];
	if (dump[i][j][G] < min)
		min = dump[i][j][R];
	if (dump[i][j][B] < min)
		min = dump[i][j][R];
}

diff = max - min;

for (i=0; i<XSIZE; i++)
for (j=0; j<YSIZE; j++)
{
	temp = (int)dump[i][j][R] - min;
	temp = (temp * 255) / diff;
	dump[i][j][R] = (char) temp;
	temp = (int)dump[i][j][G] - min;
	temp = (temp * 255) / diff;
	dump[i][j][G] = (char) temp;
	temp = (int)dump[i][j][B] - min;
	temp = (temp * 255) / diff;
	dump[i][j][B] = (char) temp;
}
lrectwrite(0,0, screen.xsize-1, screen.ysize-1, (ulong *) dump);
}


/********************************************************/
/* Saves the current display onto the file name	given	*/
/* when prompted					*/
/********************************************************/
void info_save_proc(FL_OBJECT *n, long m)
{
char *fname;

fname = fl_show_file_selector("File To Save","","*.rgb","");
if (fname != NULL)
	save_rgb_image(fname,(char *) dump, XSIZE, YSIZE);
}


/********************************************************/
/* Displays the information of the given x-y coordinate */
/* on the text fields on the form			*/
/* The x-y passed must be within the size of the data	*/
/* structure						*/
/********************************************************/
void put_info(short x, short y)
{
char text[80];

fl_freeze_form(main_form);

sprintf(text,"%d",x);
fl_set_object_label(i_x,text);

sprintf(text,"%d",y);
fl_set_object_label(i_y,text);

sprintf(text,"%d",screen.scr[x][y].max[SR]);
fl_set_object_label(i_r_max,text);

sprintf(text,"%d",screen.scr[x][y].max[SG]);
fl_set_object_label(i_g_max,text);

sprintf(text,"%d",screen.scr[x][y].max[SB]);
fl_set_object_label(i_b_max,text);

sprintf(text,"%d",screen.scr[x][y].min[SR]);
fl_set_object_label(i_r_min,text);

sprintf(text,"%d",screen.scr[x][y].min[SG]);
fl_set_object_label(i_g_min,text);

sprintf(text,"%d",screen.scr[x][y].min[SB]);
fl_set_object_label(i_b_min,text);

sprintf(text,"%d",screen.scr[x][y].tot[SR]/screen.no_frames);
fl_set_object_label(i_r_aver,text);

sprintf(text,"%d",screen.scr[x][y].tot[SG]/screen.no_frames);
fl_set_object_label(i_g_aver,text);

sprintf(text,"%d",screen.scr[x][y].tot[SB]/screen.no_frames);
fl_set_object_label(i_b_aver,text);

sprintf(text,"%d",screen.scr[x][y].tot[SR]);
fl_set_object_label(i_r_sum,text);

sprintf(text,"%d",screen.scr[x][y].tot[SG]);
fl_set_object_label(i_g_sum,text);

sprintf(text,"%d",screen.scr[x][y].tot[SB]);
fl_set_object_label(i_b_sum,text);

sprintf(text,"%d",screen.scr[x][y].max_pos[SR]);
fl_set_object_label(i_r_max_pos,text);

sprintf(text,"%d",screen.scr[x][y].max_pos[SG]);
fl_set_object_label(i_g_max_pos,text);

sprintf(text,"%d",screen.scr[x][y].max_pos[SB]);
fl_set_object_label(i_b_max_pos,text);

sprintf(text,"%d",screen.scr[x][y].min_pos[SR]);
fl_set_object_label(i_r_min_pos,text);

sprintf(text,"%d",screen.scr[x][y].min_pos[SG]);
fl_set_object_label(i_g_min_pos,text);

sprintf(text,"%d",screen.scr[x][y].min_pos[SB]);
fl_set_object_label(i_b_min_pos,text);

sprintf(text,"%d",screen.scr[x][y].complete_edge);
fl_set_object_label(i_edge,text);

fl_unfreeze_form(main_form);
}

/********************************************************/
/* Procedure is called when the mouse is clicked on the */
/* info window and puts the information specific to the */
/* pixes that is clicked on. A check is made if the 	*/
/* mouse was clicked within the window			*/
/********************************************************/
void info_callback(short dev, short val)
{
long x,xx, y,yy;

getorigin(&xx,&yy);
while(getbutton(LEFTMOUSE))
{
	x = (short) (getvaluator(MOUSEX) - xx);
	y = YSIZE - (short) (getvaluator(MOUSEY) - yy);
}

if (x >= 0 && x < XSIZE && y >= 0 && y < YSIZE)
	put_info(x,y);
}

/********************************************************/
/* Draws the data in the info window depending on the 	*/
/* domain passed. The bands are all displayed if the	*/
/* parameter all == TRUE, otherwise it checks the 	*/
/* RGB radio buttons.					*/
/********************************************************/
void draw_data(char what, char all)
{
int i, j;
char r = FALSE, g = FALSE ,b = FALSE ;

winset(i_win);
gconfig();

if (all == TRUE || fl_get_button(info_r) == 1)
	r = TRUE;
if (all == TRUE || fl_get_button(info_g) == 1)
	g = TRUE;
if (all == TRUE || fl_get_button(info_b) == 1)
	b = TRUE;

for(j= 0; j < YSIZE; j++)
{
	for(i= 0; i < XSIZE; i++)
	{
	if (what == 0)
	{
		if (r)
		assval(dump,i,j,R, screen.scr[i][j].tot[SR] / screen.no_frames);
		else
		assval(dump,i,j,R, 0);

		if (g)
		assval(dump,i,j,G, screen.scr[i][j].tot[SG] / screen.no_frames);
		else
		assval(dump,i,j,G, 0);

		if (b)
		assval(dump,i,j,B, screen.scr[i][j].tot[SB] / screen.no_frames);
		else
		assval(dump,i,j,B, 0);

		assval(dump,i,j,0, 0);
	}
	else if (what == DOM_MAXV)
	{
		if (r)
		assval(dump,i,j,R, screen.scr[i][j].max[SR]);
		else
		assval(dump,i,j,R, 0);

		if (g)
		assval(dump,i,j,G, screen.scr[i][j].max[SG]);
		else
		assval(dump,i,j,G, 0);

		if (b)
		assval(dump,i,j,B, screen.scr[i][j].max[SB]);
		else
		assval(dump,i,j,B, 0);
		assval(dump,i,j,0, 0);
	}
	else if (what == DOM_MAXP)
	{
		if (r)
		assval(dump,i,j,R, screen.scr[i][j].max_pos[SR]);
		else
		assval(dump,i,j,R, 0);

		if (g)
		assval(dump,i,j,G, screen.scr[i][j].max_pos[SG]);
		else
		assval(dump,i,j,G, 0);

		if (b)
		assval(dump,i,j,B, screen.scr[i][j].max_pos[SB]);
		else
		assval(dump,i,j,B, 0);

		assval(dump,i,j,0, 0);
		
	}
	else if (what == DOM_MINV)
	{
		if (r)
		assval(dump,i,j,R, screen.scr[i][j].min[SR]);
		else
		assval(dump,i,j,R, 0);

		if (g)
		assval(dump,i,j,G, screen.scr[i][j].min[SG]);
		else
		assval(dump,i,j,G, 0);
		if (b)
		assval(dump,i,j,B, screen.scr[i][j].min[SB]);
		else
		assval(dump,i,j,B, 0);
		assval(dump,i,j,0, 0);
	}
	else if (what == DOM_MINP)
	{
		if (r)
		assval(dump,i,j,R, screen.scr[i][j].min_pos[SR]);
		else
		assval(dump,i,j,R, 0);

		if (g)
		assval(dump,i,j,G, screen.scr[i][j].min_pos[SG]);
		else
		assval(dump,i,j,G, 0);
		if (b)
		assval(dump,i,j,B, screen.scr[i][j].min_pos[SB]);
		else
		assval(dump,i,j,B, 0);
		assval(dump,i,j,0, 0);
	}
	else if (what == DOM_EDGE)
	{
		assval(dump,i,j,R, screen.scr[i][j].sample_edge);
		assval(dump,i,j,G, screen.scr[i][j].sample_edge);
		assval(dump,i,j,B, screen.scr[i][j].sample_edge);
	}
	}
}
lrectwrite(0,0, screen.xsize-1, screen.ysize-1, (ulong *) dump);
}

/********************************************************/
/*Overlays completed edges on top of current image in   */
/*dump.							*/
/********************************************************/
void info_overlay_proc(FL_OBJECT *n, long m)
{
short x, y;

for(x = 0; x < XSIZE; x++)
{
	for(y = 0; y < YSIZE; y++)
	{
		if(screen.scr[x][y].complete_edge == 255)
		{
			assval(dump, x, y, R, 0);
			assval(dump, x, y, G, 0);
			assval(dump, x, y, B, 255);
		}
	}
}
dump_image(dump);
}

/********************************************************/
/* Sets up the gl parameters of the info window		*/
/********************************************************/
void open_info_win()
{
prefsize(screen.xsize,screen.ysize);
i_win = winopen("INFO WINDOW");
RGBmode();
pixmode(PM_TTOB, 1);
gconfig();
}
	
/********************************************************/
/* Initializes the gl and forms specific settings	*/
/* The callback for the mouse event is set up here	*/
/********************************************************/
void init_info(int callback)
{
fl_set_button(info_aver_but,1);
fl_set_button(info_r,1);
fl_set_button(info_g,1);
fl_set_button(info_b,1);
open_info_win();
draw_data(0, FALSE);
fl_init();
fl_qdevice(LEFTMOUSE);
fl_set_event_call_back(info_callback);
}

/********************************************************/
/* SELF EXPLANATORY					*/
/********************************************************/
void close_info_win()
{
winclose(i_win);
i_win = -1;
}
