/********************************************************/
/*	MAIN MULTI-SPECTRAL AYALYSIS MODULE		*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/
/*      Ideas      :    Frank Fueten, Scott Goodchild   */
/*      Programming:    Vladimir Tucakov                */
/********************************************************/

#include <stdio.h>
#include <gl.h>

#include "forms.h"
#include "gv_forms.h"
#include "basic_imm.h"
#include "types.h"
#include "ms.h"

#include "twoD.h"

#include "m_ms.h"

#define NO_OF_B_BUTS 12


extern long pers_win;

extern pointT *disp_arr;
extern char rot_dir;
extern short box;
extern int     disp_size;
extern box_T *boxing;
extern box_T *curr_box;
extern char ms_mode;
extern screenT screen;
extern thetype space1, space2, curr_space;

extern int rot_r, rot_g, rot_b;


char but_count = 0;
short curr_but = -1;
long twoDwin;

char curr_domain = 0;
char box_modified = FALSE;


FL_OBJECT *b_buts[NO_OF_B_BUTS];
FL_OBJECT *b_col[NO_OF_B_BUTS];
char but_col_index[NO_OF_B_BUTS];
box_T *boxes[NO_OF_B_BUTS];
char visible[NO_OF_B_BUTS];

/* defined arbitrarily */

char c_map[15] = {
		0,/* BLACK */
		7, /* WHITE */
		1, /* RED */
		2, /* GREEN */
		3, /* YELLOW */
		4, /* BLUE */
		5, /* VIOLET */
		6, /* L. BLUE */
		9,
		10,
		11,
		12,
		13,
		14,
		15};

/********************************************************/
/* Initializes m.s. module, the colors of the buttons	*/
/* GL stuff... It is called only once at the beginning	*/
/* of the program.					*/
/********************************************************/
void m_ms_init(char dom, short people,short place)
{
int i;
	b_buts[0] = b_but_1;
	b_buts[1] = b_but_2;
	b_buts[2] = b_but_3;
	b_buts[3] = b_but_4;
	b_buts[4] = b_but_5;
	b_buts[5] = b_but_6;
	b_buts[6] = b_but_7;
	b_buts[7] = b_but_8;
	b_buts[8] = b_but_9;
	b_buts[9] = b_but_10;
	b_buts[10] = b_but_11;
	b_buts[11] = b_but_12;
	b_col[0] = b_col_1;
	b_col[1] = b_col_2;
	b_col[2] = b_col_3;
	b_col[3] = b_col_4;
	b_col[4] = b_col_5;
	b_col[5] = b_col_6;
	b_col[6] = b_col_7;
	b_col[7] = b_col_8;
	b_col[8] = b_col_9;
	b_col[9] = b_col_10;
	b_col[10] = b_col_11;
	b_col[11] = b_col_12;
	ms_init();
	init_pers(dom, people, place);
	open_pers();
	fl_show_object(rot_grp);
	kill_boxes(&boxing);
	curr_box = boxing = NULL;
	but_count = 0;
	curr_but = -1;
	ms_mode = MAIN_MS;
	box_modified = FALSE;
}


/********************************************************/
/* FL callback proc. shows the 2D image of the 		*/
/* tresholded points					*/
/********************************************************/
void m_ms_twoD_proc(FL_OBJECT *n, long m)
{
short val, *twoD_scr;

	if (boxing != NULL)
	{ 
		twoD_main(curr_domain,curr_space);
		fprintf(stderr,"OUT OF TWOD\n");
	}
	else
	{
                fl_show_message("There have to be some boxes around",
                                "to do 2D stuff!","");
	}
}


/********************************************************/
/* FL callback proc. Turns the points highlighted by	*/
/* the box on or off					*/
/********************************************************/
void m_ms_vis_proc(FL_OBJECT *n, long m)
{
fprintf(stderr,"vis caught\n");
	if (visible[curr_but] == TRUE)
	{
		blank_box(boxes[curr_but],disp_arr,disp_size,BLACK);
		visible[curr_but] = FALSE;
	}
	else
	{
		blank_box(curr_box,disp_arr,disp_size,curr_box->color);
		visible[curr_but] = TRUE;
	}
}


/********************************************************/
/* FL callback proc. Adds a button on the form, displays*/
/* a new box in 3D and makes it the current one		*/
/********************************************************/
void m_ms_new_proc(FL_OBJECT *n, long m)
{
	if (but_count < NO_OF_B_BUTS)
	{
		box_modified = TRUE;
		fl_show_object(b_buts[but_count]);
		fl_show_object(b_col[but_count]);
		but_col_index[but_count] = c_map[1];
		if (curr_but >= 0)
			fl_set_button(b_buts[curr_but],0);
		curr_but = but_count;
		fl_set_button(b_buts[but_count],1);


		but_count++;
		new_box(&curr_box,but_col_index[curr_but]);
		boxes[curr_but] = curr_box ;
		visible[curr_but] = TRUE;
		if (boxing == NULL)
			boxing = curr_box;
	}
}



/********************************************************/
/* FL callback proc. Changes the color of the current	*/
/* box, depending on which color button was pressed	*/
/********************************************************/
void m_ms_color_proc(FL_OBJECT *n, long m) 
{
	fprintf(stderr,"argument is %d\n",m);
	if (curr_but >= 0)
	{
		fl_set_object_color(b_col[curr_but],c_map[m-1],0);
		set_box_color(boxes[curr_but],c_map[m-1]);
		but_col_index[curr_but] = c_map[m-1];
		blank_box(boxes[curr_but],
			disp_arr,disp_size,boxes[curr_but]->color);
	}
	else
	{
		fl_show_message("You must select a button in",
				"order to choose a color !","");
	}
}

/********************************************************/
/* FL callback proc. Invokes the box saving routine	*/
/********************************************************/
void m_ms_save_proc(FL_OBJECT *n, long m)
{
char *fname;

	fname = fl_show_file_selector("Boxing Info To Save","","","");
	if (fname == NULL)
		return;
	save_box(fname, boxing);
	box_modified = FALSE;
}


/********************************************************/
/* Displays the buttons associated with the defined	*/
/* boxes						*/
/********************************************************/
void display_buts()
{
box_T *temp;

        temp = boxing;
        but_count = 0;
        while (temp != NULL)
        {
                but_count++;
                curr_but = but_count-1;

                fl_show_object(b_buts[curr_but]);
                fl_show_object(b_col[curr_but]);
                but_col_index[curr_but] = temp->color;
                curr_box = temp;
                boxes[curr_but] = temp;
                fl_set_object_color(b_col[curr_but],temp->color,0);
                blank_box(curr_box, disp_arr, disp_size, curr_box->color);
                visible[curr_but] = TRUE;
                temp = temp->next;
        }
        fl_set_button(b_buts[but_count],1);

}


/********************************************************/
/* FL callback proc. Invokes the box data structure 	*/
/* loading procedure.					*/
/********************************************************/
void m_ms_load_proc(FL_OBJECT *n, long m)
{
char *fname;
box_T *temp;

	
	fname = fl_show_file_selector("Boxing Info To Load","","","");
	if (fname == NULL)
		return;
	load_box(fname, &boxing, &curr_box);
	display_buts();
}

/********************************************************/
/* FL callback proc. Changes the current box depending 	*/
/* on which box button is pressed.			*/
/********************************************************/
void b_but_proc(FL_OBJECT *n, long m)
{
	curr_but = m - 1;
	curr_box = boxes[curr_but];
	fprintf(stderr,"current but = %d \n",curr_but);
}

/********************************************************/
/* Open the GL window for displaying the 3D m.s. data	*/
/********************************************************/
void open_pers()
{

        prefsize(700,700);
        pers_win = winopen("spin");
	winset(pers_win);
        mmode(MVIEWING);
        perspective(V_FOV, 1, 50, 1000);
        lookat(0,00,600,0,0,0,0);
        doublebuffer();
        gconfig();

        color(BLACK);
        clear();
        swapbuffers();
        color(BLACK);
        clear();
}


/********************************************************/
/* if the boxes were modified the routine Asks weather 	*/
/* the boxes should be saved				*/
/********************************************************/
void box_confirm()
{
	if (box_modified == TRUE && 
		fl_show_question("BOXES MODIFIED",
			"Do you want to save the new boxes?",""))
	{
		m_ms_save_proc(NULL, 0);
		fprintf(stderr,"SAVING BOXES !!! :):):):)\n");
	}
}


/********************************************************/
/* The main event loop of the form. Mainly in charge of	*/
/* the rotation						*/
/********************************************************/
void m_main()
{
int angle=1;
FL_OBJECT *cmd;
char c= ROT_G;

	angle = 0;
        put_box();
        for(;;)
        {
                cmd = fl_check_forms();
                if (cmd == m_ms_done)
                {
                        free(disp_arr);
			disp_arr=NULL;
                        winclose(pers_win);
			box_confirm();
			kill_space(space1);
			space1 = NULL;
			kill_space(space2);
			space2 = NULL;
                        fl_hide_all();
                        fl_show_object(main_grp);
			return;
                }
		else if (cmd == m_ms_def)
		{
			if (curr_but >= 0)
			{
				box_modified = TRUE;
				fl_hide_all();
				fl_show_object(ms_grp);
			fl_show_object(rot_grp);
				ms_mode = DEF_MS;
				blank_box(boxes[curr_but],disp_arr,
					disp_size,WHITE);
				ms_main();
			}
			else
			{
				fl_show_message("You must select a button in",
						"order to choose a color !","");
			}
		}
                else if (cmd == ms_red_but)
                        rot_dir = ROT_R;
                else if (cmd == ms_green_but)
                        rot_dir = ROT_G;
                else if (cmd == ms_blue_but)
                        rot_dir = ROT_B;
                else if (cmd == ms_slider)
                        angle = (int)fl_get_slider_value(ms_slider);
                else if (cmd == ms_axes)
                {
                        if (box == 1)
                                box = 0;
                        else
                                box = 1;
                }

		if(rot_dir == ROT_R)
                        rot_r += angle;
                else if (rot_dir == ROT_G)
                        rot_g += angle;
                else if (rot_dir == ROT_B)
                        rot_b += angle;

                rotate(angle, rot_dir);
                display_data_2(disp_arr,disp_size,boxes[curr_but]);

        }
}

/********************************************************/
/* This procedure is called when coming back from 	*/
/* the box definition or twoD form.			*/
/********************************************************/
void reenter_m_ms()
{
int i;

	fl_hide_all();
	fl_show_object(ms_main_grp);
	fl_show_object(rot_grp);
	ms_mode = MAIN_MS;
	if (visible[curr_but] == TRUE)
	{
		blank_box(curr_box,disp_arr,disp_size,curr_box->color);
	}
	else
	{
		blank_box(curr_box,disp_arr,disp_size,BLACK);
	}
	for(i=0; i<but_count; i++)
	{
                fl_show_object(b_buts[i]);
                fl_show_object(b_col[i]);
	}
}

/********************************************************/
/* The following code implements the level of forms in	*/
/* which it is specified in what domain the ms is to be	*/
/* done and what kind of filtering is to be applied	*/
/********************************************************/


/********************************************************/
/* initializes the form, called only at the beginning 	*/
/* of the program					*/
/********************************************************/
void mroot_init()
{
	fl_set_counter_bounds(mroot_people,1,100);
	fl_set_counter_value(mroot_people,1);
	fl_set_counter_step(mroot_people, 1,0);
	fl_set_counter_precision(mroot_people, 0);

	fl_set_counter_bounds(mroot_place,1,3);
	fl_set_counter_value(mroot_place,1);
	fl_set_counter_step(mroot_place, 1,0);
	fl_set_counter_precision(mroot_place, 0);
}

/********************************************************/
/* FL callback proc. sets the current domain		*/
/* depending on which button is pressed			*/
/********************************************************/
void mroot_dom_proc(FL_OBJECT *n, long m)
{
	curr_domain = m;
fprintf(stderr,"curr dom = %d\n", curr_domain);
}

/********************************************************/
/* cancels ms and returns to the main form		*/
/********************************************************/
void mroot_can_proc(FL_OBJECT *n, long m)
{
	fl_hide_all();
	fl_show_object(main_grp);
}

/********************************************************/
/* starts the ms analysis based on the selection on 	*/
/* the form						*/
/********************************************************/
void mroot_ready_proc(FL_OBJECT *n, long m)
{
	fl_hide_all();
	if (fl_get_button(mroot_filter) == 1)
		m_ms_init(curr_domain,
				fl_get_counter_value(mroot_people),
				fl_get_counter_value(mroot_place));
	else
		m_ms_init(curr_domain,0,0);
	fl_show_object(ms_main_grp);
	m_main();
}

