/********************************************************/
/*	Geo Vision					*/
/*	A very cool program for a modern geologist	*/
/********************************************************/
/*	Copyright, 1995					*/
/*	Ph.D. Frank Fueten				*/
/*	Department of Earth Sciences			*/
/*	Brock University				*/
/*	St. Catharines, Ontario, Canada			*/
/********************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include <dmedia/vl.h>
#include "image.h"
#include "main.h"
#include "forms.h"
#include "gv_forms.h"
#include "types.h"

extern screenT screen;
extern char two_dump[XSIZE][YSIZE][3];

extern void open_serial_port();
extern void close_serial_port();
extern void turn_off_echo();

long measure_win = -1;
long id_win = -1;
long overlay_win = -1;

/********************************************************/
/* This procedure hides all groups of objects defined	*/
/* in forms. It is used when swithching between menues	*/
/********************************************************/
fl_hide_all()
{
fl_hide_object(main_grp);
fl_hide_object(file_grp);
fl_hide_object(samp_grp);
fl_hide_object(info_grp);
fl_hide_object(ms_main_grp);
fl_hide_object(ms_grp);
fl_hide_object(stage_grp);
fl_hide_object(b_buts_grp);
fl_hide_object(rot_grp);
fl_hide_object(filter_grp);
fl_hide_object(indx_grp);
fl_hide_object(ms_root_grp);
fl_hide_object(samp_save_grp);
fl_hide_object(edge_grp);
fl_hide_object(hist_grp);
fl_hide_object(measure_grp);
fl_hide_object(id1_grp);
fl_hide_object(id2_grp);
fl_hide_object(camera_grp);
}
	
/********************************************************/
/* Starts the info menu if there is a sample		*/
/********************************************************/
void m_info_proc(FL_OBJECT *n, long m)
{
if (screen.exists == FALSE)
{
	fl_show_message("NO SAMPLE INFORMATION !","","Need to use file or sample first.");
}

else
{
	fl_hide_all();
	init_info();
	fl_show_object(info_grp);
}
}

/********************************************************/
/* Initializes the choices of the choice menu in the	*/
/* rotation menu and histogram menu			*/
/********************************************************/
void init_rot()
{
fl_addto_choice(rot_def_pos, "STRAIGHT");
fl_addto_choice(rot_def_pos, "BACK");
fl_addto_choice(rot_def_pos, "TOP");
fl_addto_choice(rot_def_pos, "BOTTOM");
fl_addto_choice(rot_def_pos, "RIGHT");
fl_addto_choice(rot_def_pos, "LEFT");
fl_addto_choice(rot_def_pos, "DIAGONAL");

fl_addto_choice(hist_def_col, "RGB");
fl_addto_choice(hist_def_col, "RED");
fl_addto_choice(hist_def_col, "GREEN");
fl_addto_choice(hist_def_col, "BLUE");
}

/********************************************************/
/* Starts the ms analysis menu if there is a sample     */
/********************************************************/
void m_ms_proc(FL_OBJECT *n, long m)
{
if(screen.exists == FALSE)
{
	fl_show_message("NO SAMPLE INFORMATION !","",
			"Need to use file or sample first.");
}

else
{
	fl_hide_all();
	fl_show_object(ms_root_grp);
}
}

/********************************************************/
/*Starts the camera menu.				*/
/********************************************************/
void m_camera_proc(FL_OBJECT *n, long m)
{
fl_hide_object(main_grp);
fl_set_button(c_which_polars_but, 1);
fl_show_object(camera_grp);
}

/********************************************************/
/* Starts the file menu 				*/
/********************************************************/
void m_file_proc(FL_OBJECT *n, long m)
{
fl_hide_all();
fl_show_object(file_grp);
}

/********************************************************/
/* Starts id menu					*/
/********************************************************/
void m_id_proc(FL_OBJECT *n, long m)
{
if (screen.exists == FALSE)
{
	fl_show_message("NO SAMPLE INFORMATION !","","Need to use file or sample first.");
}

else
{
	fl_hide_object(main_grp);
	fl_show_object(id1_grp);
	fl_show_object(id2_grp);

	prefsize(XSIZE, YSIZE);
	id_win = winopen("ID WINDOW");
	RGBmode();
	pixmode(PM_TTOB, 1);
	gconfig();

	prefsize(XSIZE, YSIZE);
	overlay_win = winopen("OVERLAY WINDOW -- CLICK IN HERE");
	RGBmode();
	pixmode(PM_TTOB, 1);
	gconfig();

	put_overlay_pixels_proc(NULL, 0);

	fl_init();
	fl_qdevice(LEFTMOUSE);
	fl_set_event_call_back(id_dummy_callback);

	fl_set_button(id_max_button, 1);
	fl_set_button(id_min_button, 0);
	fl_set_button(id_average_button, 0);
	fl_set_button(id_min_pos_button, 0);
	fl_set_button(id_max_pos_button, 0);

	overlay_finished();
}
}

/********************************************************/
/* The edge menu is temporarily hooked up to this button*/
/********************************************************/
void m_edge_proc(FL_OBJECT *n, long m)
{
if(screen.exists == FALSE)
{
	fl_show_message("NO SAMPLE INFORMATION !","",
			"Need to use file or sample first.");
}
       
else
{
	fl_hide_all();
	fl_show_object(edge_grp);
	fl_init();
	fl_qdevice(LEFTMOUSE);
	fl_set_event_call_back(id_dummy_callback);
	start_edge();
}
}

/********************************************************/
/*Peforms inertia tensor and C-axis measurements.       */
/********************************************************/
void m_measure_proc(FL_OBJECT *n, long m)
{
if(screen.exists == FALSE)
{
	fl_show_message("NO SAMPLE INFORMATION !","",
			"Need to use file or sample first.");
}

else
{
	fl_hide_all();
	fl_set_button(m_display_edge_but, 1);
	fl_set_button(m_display_max_but, 0);
	fl_set_button(m_display_maxp_but, 0);
	fl_show_object(measure_grp);

	prefsize(XSIZE, YSIZE);
	measure_win = winopen("MEASURE WINDOW");
	RGBmode();
	pixmode(PM_TTOB, 1);
	gconfig();
	put_edge_pixels();
	dump_image(two_dump);
}
}

/********************************************************/
/* Starts the sampling menu				*/
/********************************************************/
void m_samp_proc(FL_OBJECT *n, long m)
{
fl_hide_all();
fl_show_object(samp_grp);
fl_set_input(num_image_average, "2");
fl_set_button(cross_pol_sample_but, 0);
fl_set_button(plane_pol_sample_but, 0);
fl_set_button(both_pol_sample_but, 1);
}

/********************************************************/
/* Starts the stage control menu			*/
/********************************************************/
void m_stage_proc(FL_OBJECT *n, long m)
{
fl_hide_all();
stg_init();
fl_show_object(stage_grp);
fl_show_object(filter_grp);
}

/********************************************************/
/* The main procedure of the application. Initializes	*/
/* the forms and data structures and lets the form lib	*/
/* take over						*/
/********************************************************/
void main(int argc, char **argv)
{
FL_OBJECT *obj;

open_serial_port();

turn_off_echo();

create_the_forms();

fl_set_input(caxes_plunge_input,"0");

fl_set_input(neigh_num_input, "3");
fl_set_input(pix_level_input, "22");

fl_set_input(edge_thres_low, "40");
fl_set_input(edge_thres_high, "150");

fl_set_input(grid_input_x, "10");
fl_set_input(grid_input_y, "10");

mroot_init();
threshold_init();
init_min_name();
init_rot();
init_s_choices();

fl_hide_all();
fl_show_object(main_grp);

fl_show_form(main_form,FL_PLACE_CENTER,TRUE,"GeoVision");

do obj = fl_do_forms(); while (obj != m_done );
     
close_serial_port();
gexit();	
}
