/********************************************************/
/*      MULTI-SPECTRAL AYALYSIS MODULE             	*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/
/*      Ideas      :    Frank Fueten, Scott Goodchild   */
/*      Programming:    Vladimir Tucakov                */
/********************************************************/

#include <gl/gl.h>
#include <stdio.h>
#include <gl/device.h>
#include "image.h"
#include <malloc.h>
#include "forms.h"
#include "gv_forms.h"
#include "basic_imm.h"
#include "manual.h"
#include "ms.h"

#define GL_SIZE	2
#define V_FOV	400

#define COUNTER_MIN 0.0
#define COUNTER_MAX 255.0
#define COUNTER_S_STEP 2
#define COUNTER_L_STEP 10


   extern int density;
long win1,win2;
extern long twoDwin;
long pers_win;

extern box_T *boxing;
extern box_T *curr_box;
extern unsigned short man_color;


char rot_dir = ROT_G;
char ms_exit = FALSE;
pointT *disp_arr;
int     disp_size;
int xsize, ysize;
int sp_1,sp_2;
char ms_mode;

thetype space1 = NULL, space2 = NULL, curr_space;

short box=0;

int rot_r = 0, rot_g = 0, rot_b = 0;

/********************************************************/
/* THE FOLLOWING ARE THE COORDINATE DEFINITIONS OF 	*/
/* THE 3D-RGB-GL SPACE VERTECIES			*/
/********************************************************/
float center[3] = {0,0,0}, 
	max_red[3]={RGB_AXIS_SIZE,0,0},
        max_green[3]={0,RGB_AXIS_SIZE,0}, 
	max_blue[3]={0,0,RGB_AXIS_SIZE};

float very_far[3] = {RGB_AXIS_SIZE,RGB_AXIS_SIZE,RGB_AXIS_SIZE},
        rg_far[3] = {RGB_AXIS_SIZE,RGB_AXIS_SIZE,0},
	rb_far[3] ={RGB_AXIS_SIZE,0,RGB_AXIS_SIZE},
        gb_far[3] = {0,RGB_AXIS_SIZE,RGB_AXIS_SIZE};

short old_r_min, old_r_max, old_g_min, old_g_max, old_b_min, old_b_max;


/********************************************************/
/* initializes the FL values, the sliders...		*/
/********************************************************/
void ms_init()
{
fl_set_slider_bounds(ms_slider,-10,10);
fl_set_slider_value(ms_slider,0);
fl_set_counter_bounds(ms_r_max,COUNTER_MIN,COUNTER_MAX);
fl_set_counter_bounds(ms_r_min,COUNTER_MIN,COUNTER_MAX);
fl_set_counter_bounds(ms_g_max,COUNTER_MIN,COUNTER_MAX);
fl_set_counter_bounds(ms_g_min,COUNTER_MIN,COUNTER_MAX);
fl_set_counter_bounds(ms_b_max,COUNTER_MIN,COUNTER_MAX);
fl_set_counter_bounds(ms_b_min,COUNTER_MIN,COUNTER_MAX);

fl_set_counter_step(ms_r_max, COUNTER_S_STEP,COUNTER_L_STEP);
fl_set_counter_step(ms_r_min, COUNTER_S_STEP,COUNTER_L_STEP);
fl_set_counter_step(ms_g_max, COUNTER_S_STEP,COUNTER_L_STEP);
fl_set_counter_step(ms_g_min, COUNTER_S_STEP,COUNTER_L_STEP);
fl_set_counter_step(ms_b_max, COUNTER_S_STEP,COUNTER_L_STEP);
fl_set_counter_step(ms_b_min, COUNTER_S_STEP,COUNTER_L_STEP);

fl_set_counter_precision(ms_r_max, 0);
fl_set_counter_precision(ms_r_min, 0);
fl_set_counter_precision(ms_g_max, 0);
fl_set_counter_precision(ms_g_min, 0);
fl_set_counter_precision(ms_b_max, 0);
fl_set_counter_precision(ms_b_min, 0);
}

/********************************************************/
/* Initializes the 3D space data structure by making 	*/
/* the necessary spaces filling it with the appropriate	*/
/* domain and filtering depending of the parameters	*/
/* passed. Filtering is done if parameter place is	*/
/* greater than zero.					*/
/********************************************************/
void init_pers(char dom, short people, short place)
{
int mama;
char ans, quest[80];
int thresh;

fprintf(stderr,"space1 = %d\n\n",space1);
	if (space1 == NULL)
		make_space(&space1);
fprintf(stderr,"space1 = %d\n\n",space1);

        sp_1 = fill_space(space1,dom);
	thresh = (int) fl_get_counter_value(mroot_thresh);
	if (thresh >0)
		ms_threshold(space1,thresh);
	if (fl_get_button(mroot_filter) ==1)
	{
		make_space(&space2);
		sp_2 = kill_lonely(space1, space2, people, place);
		disp_size = prepare_display(space2, &disp_arr,sp_2);
		curr_space = space2;
	}
	else
	{
		disp_size = prepare_display(space1, &disp_arr,sp_1);
		curr_space = space1;
	}
        fprintf(stderr,"the image dispalyed has %d pixels\n",disp_size);
/*
	kill_space(space1);
*/
}

/********************************************************/
/* The points in 3D space are displayed by using a 	*/
/* separate data structure. This is done because there	*/
/* is very few points that need to be displayed as 	*/
/* compared to the size of the 3D space.		*/
/* The display data structure contains the coordinates	*/
/* and the color of the points that should be displayed	*/
/********************************************************/
int prepare_display(thetype space, pointT **disp, int somenumber)
{

int i,j,k;
pointT *temp;
int counter = 0;
char text[80];
char per = 0;
float ratio;

ratio = (float)TOT_AXIS_SIZE/(float)RGB_AXIS_SIZE;

fprintf(stderr,"the number of points is %d \n", somenumber);
temp = calloc(somenumber,sizeof(pointT));

if (temp == NULL)
{
	fprintf(stderr,"Memory allocation problem. EXITING PROGRAM !\n");
	exit(0);
}

fprintf(stderr,"preparing the DISPLAY\n");
fprintf(stderr,"Some number is = %d\n",somenumber);

  fl_hide_object(go);
  fl_show_object(progress);
  sprintf(text,"Converted %d\% !",per);
  fl_set_object_label(progres_text,text);
  fl_show_form(samp_info,FL_PLACE_CENTER,FALSE,NULL);
  

for (i = 0 ; i< TOT_SPACE_SIZE; i++)
{
  per = i*100/TOT_SPACE_SIZE;
  sprintf(text,"Converted %d % !",per);sprintf(text,"Converted %d % !",per);
  fl_set_object_label(progres_text,text);
fprintf(stderr,"%d/255 DONE I added this...\r",i);
        for (j = 0 ; j< TOT_SPACE_SIZE; j++)
        {
                for (k = 0 ; k< TOT_SPACE_SIZE; k++)
                {

                        if (get_from_space(space,i,j,k) >0)
                        {
                                temp[counter].val[0] = ((float)i)/ratio;
                                temp[counter].val[1] = ((float)j)/ratio;
                                temp[counter].val[2] = ((float)k)/ratio;
                                temp[counter].color = WHITE;
                                counter++;
                        }
                }
        }
}
  fl_hide_form(samp_info);
fprintf(stderr,"final count is %d \n",counter);
*disp = temp;
return(counter);

}


/********************************************************/
/* puts the box that is the border of the coordinate	*/
/* system						*/
/********************************************************/
void put_box()
{
        color(RED);
        bgnline();
                put_point(max_green);
                put_point(rg_far);
        endline();
        bgnline();
                put_point(gb_far);
                put_point(very_far);
        endline();
        bgnline();
                put_point(max_blue);
                put_point(rb_far);
        endline();
        color(GREEN);
        bgnline();
                put_point(max_red);
                put_point(rg_far);
        endline();
        bgnline();
                put_point(rb_far);
                put_point(very_far);
        endline();
        bgnline();
                put_point(max_blue);
                put_point(gb_far);
        endline();
        color(BLUE);
        bgnline();
                put_point(max_red);
                put_point(rb_far);
        endline();
        bgnline();
                put_point(rg_far);
                put_point(very_far);
        endline();
        bgnline();
                put_point(max_green);
                put_point(gb_far);
        endline();
}

/********************************************************/
/* GL routine that displays all the points in the 3D	*/
/* space. It also checks the colors of the points,	*/
/* puts the coordinated box and the tresholding box	*/
/********************************************************/
void display_data_2(pointT *space, int size, box_T *l_box)
{
int i;
float X[3];
        color(BLACK);
        clear();
        if (l_box != NULL)
	{
                put_little_box(l_box);
	}
        color(GREEN);
        bgnline();
                put_point(center);
                put_point(max_green);
        endline();
        color(RED);
        bgnline();
                put_point(center);
                put_point(max_red);
        endline();
        color(BLUE);
        bgnline();
                put_point(center);
                put_point(max_blue);
        endline();
        if (box==0)
                put_box();

        bgnpoint();
        for(i= 0; i< size; i++)
        {
        if(space[i].color != BLACK)
        {
                color( space[i].color);
                X[0] = space[i].val[0];
                X[1] = space[i].val[1];
                X[2] = space[i].val[2];
                if(ms_mode == DEF_MS && l_box != NULL &&
                        (X[0]>=l_box->r_min) && (X[0]<=l_box->r_max) &&
                        (X[1]>=l_box->g_min) && (X[1]<=l_box->g_max) &&
                        (X[2]>=l_box->b_min) && (X[2]<=l_box->b_max))
                        {
                                        color(l_box->color);
                                        put_point(X);
                        }
                else
			put_point(X);
        }
        }
        endpoint();
        swapbuffers();
}


/********************************************************/
/* Main event loop in charge of handeling interactive	*/
/* rotation of the 3D space.				*/
/********************************************************/
int pers_view()
{
int angle=1;
FL_OBJECT *cmd;
short val;
char c= ROT_G;
int redisplay = TRUE;
float temp;

        angle = 0;

/* BOXING STUFF */


        put_box();

        for(;;)
        {
                cmd = fl_check_forms();
                if (cmd == ms_done)
                {
			reenter_m_ms();
                        return(1);
                }
                else if (cmd == ms_red_but)
                        rot_dir = ROT_R;
                else if (cmd == ms_green_but)
                        rot_dir = ROT_G;
                else if (cmd == ms_blue_but)
                        rot_dir = ROT_B;
                else if (cmd == ms_slider)
                        angle = (int)fl_get_slider_value(ms_slider);
		else if (cmd == ms_axes)
		{
			if (box == 1)
				box = 0;
			else
				box = 1;
		}
		else if (cmd == ms_r_min)
		{
			if (fl_get_button(r_lock) == 1)
			{
			temp = fl_get_counter_value(ms_r_max) - 
			  (old_r_min - fl_get_counter_value(ms_r_min));
				old_r_max = temp;
				fl_set_counter_value(ms_r_max, temp);
			}
			old_r_min = fl_get_counter_value(ms_r_min);
			b_move(curr_box);
		}
		else if (cmd == ms_r_max)
		{
			if (fl_get_button(r_lock) == 1)
			{
			temp = fl_get_counter_value(ms_r_min) - 
			  (old_r_max - fl_get_counter_value(ms_r_max));
				old_r_min = temp;
				fl_set_counter_value(ms_r_min, temp);
			}
			old_r_max = fl_get_counter_value(ms_r_max);
			b_move(curr_box);
		}
		else if (cmd == ms_g_min)
		{
			if (fl_get_button(g_lock) == 1)
			{
			temp = fl_get_counter_value(ms_g_max) - 
			  (old_g_min - fl_get_counter_value(ms_g_min));
				old_g_max = temp;
				fl_set_counter_value(ms_g_max, temp);
			}
			old_g_min = fl_get_counter_value(ms_g_min);
			b_move(curr_box);
		}
		else if (cmd == ms_g_max)
		{
			if (fl_get_button(g_lock) == 1)
			{
			temp = fl_get_counter_value(ms_g_min) - 
			  (old_g_max - fl_get_counter_value(ms_g_max));
				old_g_min = temp;
				fl_set_counter_value(ms_g_min, temp);
			}
			old_g_max = fl_get_counter_value(ms_g_max);
			b_move(curr_box);
		}
		else if (cmd == ms_b_min)
		{
			if (fl_get_button(b_lock) == 1)
			{
			temp = fl_get_counter_value(ms_b_max) - 
			  (old_b_min - fl_get_counter_value(ms_b_min));
				old_b_max = temp;
				fl_set_counter_value(ms_b_max, temp);
			}
			old_b_min = fl_get_counter_value(ms_b_min);
			b_move(curr_box);
		}
		else if (cmd == ms_b_max)
		{
			if (fl_get_button(b_lock) == 1)
			{
			temp = fl_get_counter_value(ms_b_min) - 
			  (old_b_max - fl_get_counter_value(ms_b_max));
				old_b_min = temp;
				fl_set_counter_value(ms_b_min, temp);
			}
			old_b_max = fl_get_counter_value(ms_b_max);
			b_move(curr_box);
		}
			

		if(rot_dir == ROT_R)
			rot_r += angle;
		else if (rot_dir == ROT_G)
                        rot_g += angle;
		else if (rot_dir == ROT_B)
                        rot_b += angle;
                rotate(angle, rot_dir);

                display_data_2(disp_arr,disp_size,curr_box);

        }
}

/********************************************************/
/* Entry routind of this module				*/
/********************************************************/
void ms_main()
{
	set_box_counters(curr_box);
	ms_init();
	pers_view();
}

/********************************************************/
/* FL callback proc. resets the default position of the	*/
/* view.						*/
/********************************************************/
void rot_def_pos_proc(FL_OBJECT *n, long m)
{
int r=0,g=0,b=0,choi;

	choi = fl_get_choice(rot_def_pos);

	if (choi == 1)		/* STRAIGHT */
	{}
	else if (choi == 2)	/* BACK	*/
		g = 1800;
	else if (choi == 3)	/* TOP	*/
		r = 900;
	else if (choi == 4)	/* BOTTOM */
		r = -900;
	else if (choi == 5)	/* RIGHT */
		g = -900;
	else if (choi == 6)	/* LEFT */
		g = 900;
	else if (choi == 7)	/* DIAGONAL */
		g = 1350;
		
	rotate(-1*rot_r,ROT_R);
	rotate(-1*rot_g,ROT_G);
	rotate(-1*rot_b,ROT_B);
	rotate(r,ROT_R);
	rotate(g,ROT_G);
	rotate(b,ROT_B);
	rot_r = r;
	rot_g = g;
	rot_b = b;
	display_data_2(disp_arr,disp_size,curr_box);

}
