#include <fcntl.h>
#include <termio.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "forms.h"
#include "gv_forms.h"
#include "types.h"
#include "stage.h"
 
extern char *dataPtr;
extern xsize;
extern ysize;

static short step;
short serial_port;
static char direction;

short up_polar_pos = 0;
short low_polar_pos = 0;
char stg_lock_mode = FALSE;

char command[10];

/****************************************************************/
/*Turns off terminal echo if it is on.				*/
/****************************************************************/
void turn_off_echo()
{
struct termio myio;

ioctl(serial_port, TCGETA, &myio);
if(myio.c_lflag & ECHO)   /*echo is on*/
{
	myio.c_lflag = myio.c_lflag ^ ECHO; 
	ioctl(serial_port, TCSETA, &myio); /*turn echo off*/
} 

/*set baud to 4800*/
/*
ioctl(serial_port, TCGETA, &myio);
if(myio.c_cflag & B9600)   
{
	myio.c_cflag = myio.c_cflag ^ B9600; 
	myio.c_cflag = myio.c_cflag ^ B4800; 
	ioctl(serial_port, TCSETA, &myio); 
} 
*/
}

/****************************************************************/
/*Collects the sum of red, green and blue components of a 	*/
/*single image. Used in crossing polars.			*/
/****************************************************************/
unsigned long collect_rgb_data() 
{
unsigned long sum_red, sum_green, sum_blue;
short x, y;
 
get_one_image();
dump_image(dataPtr);	

sum_red = 0;
sum_blue = 0;
sum_green = 0;

for(x = 0; x < xsize; x++) 
{              
	for(y = 0; y < ysize; y++) 
	{               
		sum_red += (unsigned long)valat(dataPtr, x, y, R); 
		sum_green += (unsigned long)valat(dataPtr, x, y, G); 
		sum_blue += (unsigned long)valat(dataPtr, x, y, B); 
	}
}
return(sum_red + sum_green + sum_blue);
}

/****************************************************************/
/*Writes a command to the serial port. Used for rotating the 	*/
/*motors that control the stage and the polarizers.		*/
/****************************************************************/
void write_to_serial_port(char *which_motor, char which_direction, short num_steps)
{
if(strlen(which_motor) == 1)
{
	sprintf(command, "%s%c%d\r", which_motor, which_direction, num_steps);
}

else if (strlen(which_motor) == 2)
{
	sprintf(command, "%s\r", which_motor);
}

else
{
	fprintf(stderr,"Something wrong with the which_motor length\n");
	exit(0);
}

if(write(serial_port, command, strlen(command) ) == -1)
{
        fprintf(stderr,"\nError writing to output port -- Aborting.\n");
        close_serial_port();
        close_video();
        exit(1);
}
}

/****************************************************************/
/*Reads in a command from the serial port. This is the response */
/*of the electronics to the commands it gets from write_to...	*/
/****************************************************************/
void read_from_serial_port()
{
char in[MAX_RESPONSE];
short insize;

/*
for(i = 0; i < MAX_RESPONSE; i++)
{
	in[i] = NULL;
}
*/

while( (insize = read(serial_port, in, MAX_RESPONSE) ) < 0);

/*
while(TRUE)
{	
	while( (insize = read(serial_port, in, MAX_RESPONSE) ) < 0);
	if(strncmp(command, in, strlen(command)) == 0)
		break;
}	
fprintf(stderr,"%s\n", in);
*/
}

/****************************************************************/
/*Opens the serial port for communication.			*/
/****************************************************************/
void open_serial_port()
{
if( (serial_port = open("/dev/ttyd1", O_RDWR|O_NDELAY|O_NONBLOCK|O_NOCTTY) ) == -1)
{
	fprintf(stderr,"Unable to open serial port -- Aborting.\n");
	exit(1);
} 
}

/****************************************************************/
/*Rotates stage to the desired filter positon then reads in 	*/
/*response from electronics.					*/
/****************************************************************/
void stage_command(char *which_filter)
{
char in[MAX_RESPONSE];
short insize;

/*clear serial port*/
while( (insize = read(serial_port, in, MAX_RESPONSE) ) > 0);

write_to_serial_port(which_filter, NULL, 0);
read_from_serial_port();
}

/****************************************************************/
/*Rotates both upper and lower polarizers, separately or 	*/
/*together. Rotates them 'num_steps' steps.			*/
/****************************************************************/
void rotate_polarizer(char *which_motor, short num_steps)
{
char in[MAX_RESPONSE];
short insize;

/*clear serial port*/
while( (insize = read(serial_port, in, MAX_RESPONSE) ) > 0);

if(num_steps != 0) 
{
	if(num_steps > 0)
  	{
		write_to_serial_port(which_motor, ROTATE_RIGHT, num_steps);	
		read_from_serial_port();
  	}
	
	if(num_steps < 0)
	{
		num_steps = abs(num_steps);
		write_to_serial_port(which_motor, ROTATE_LEFT, num_steps);
		read_from_serial_port();
	}
}
}

/****************************************************************/
/*Closes the serial port.					*/
/****************************************************************/
void close_serial_port()
{
close(serial_port);
}

/****************************************************************/
/*Crosses the polarizers by finding the orientation of the 	*/
/*polars that provides the 'darkest' image.			*/
/****************************************************************/
void cross_polars(char *which_polar)
{
short best_position, q;
unsigned long  new_rgb_sum, old_rgb_sum, best_sum, last_sum[4];

open_video();
step = 20; /*18 degrees = 20 steps*/

old_rgb_sum = collect_rgb_data();
rotate_polarizer(which_polar, step); 
/*
sleep(2);
*/
do
{
	if(step == 1)
	{
		best_sum = old_rgb_sum;

		for(q = 0; q <= 3; q++)
		{
			last_sum[q] = new_rgb_sum;
	
			if(q != 3)
			{
				rotate_polarizer(which_polar, step);
				/*sleep(2);*/
			}
	
			new_rgb_sum = collect_rgb_data();
			
			if(last_sum[q] < best_sum)
			{
				best_sum = last_sum[q];
				best_position = q;		
			}
		
		}
		

	step = abs(3 - best_position);
	change_direction();
	rotate_polarizer(which_polar, step);
	step = 0;
	}      
	
	else 
	{
		new_rgb_sum = collect_rgb_data(); 

		if(new_rgb_sum <= old_rgb_sum) /*got darker BETTER */
		{
			rotate_polarizer(which_polar, step);
			/*sleep(2);*/
		}

		else /*got lighter WORSE*/
		{
			change_direction();
			step = step / 2;
			rotate_polarizer(which_polar, step);
			/*sleep(2);*/
		}	

		old_rgb_sum = new_rgb_sum;
	}
}
while(abs(step) > 0);
close_video();
}

/****************************************************************/
/*Changes the direction of rotation of the polarizer in question*/
/*Used in crossing polarizers.					*/
/****************************************************************/
void change_direction()
{

if(step > 0)
{
	step = -step;
}

else
{
	step = abs(step);
}
}

/****************************************************************/
/*Code from this point on (in this module) delas with the stage */
/*form.								*/
/****************************************************************/

/****************************************************************/
/*Initializes polarizer counters.				*/
/****************************************************************/
void stg_init()
{

fl_set_counter_bounds(stg_up_count,STG_COUNT_MIN,STG_COUNT_MAX);
fl_set_counter_bounds(stg_low_count,STG_COUNT_MIN,STG_COUNT_MAX);

fl_set_counter_step(stg_up_count, STG_S_STEP,STG_L_STEP);
fl_set_counter_step(stg_low_count, STG_S_STEP,STG_L_STEP);

fl_set_counter_precision(stg_up_count, 0);
fl_set_counter_precision(stg_low_count, 0);
}

/****************************************************************/
/*Locks polarizers so they rotate together.			*/
/****************************************************************/
void stg_lock_proc(FL_OBJECT *n, long m)
{

if(stg_lock_mode)
{
	stg_lock_mode = FALSE;
}

else
{
	stg_lock_mode = TRUE; 
}
}

/****************************************************************/
/*Rotates upper polarizer by the amount indicated.		*/
/****************************************************************/
void stg_up_count_proc(FL_OBJECT *n, long m)
{
float temp;
short step;
char dir = ROTATE_RIGHT;

temp = fl_get_counter_value(stg_up_count);
step = (short)temp - up_polar_pos;
up_polar_pos += step;

if(stg_lock_mode)
{
	low_polar_pos += step;
	fl_set_counter_value(stg_low_count, low_polar_pos);
	rotate_polarizer(BOTH_POLARS, step);
}

else
{
	rotate_polarizer(UPPER_POLAR, step);
}
}

/****************************************************************/
/*Rotates lower polarizer by the amount indicated.		*/
/****************************************************************/
void stg_low_count_proc(FL_OBJECT *n, long m)
{
float temp;
short step;
char dir = ROTATE_RIGHT;

temp = fl_get_counter_value(stg_low_count);
step = (short)temp - low_polar_pos;
low_polar_pos += step;

if(stg_lock_mode)
{
	up_polar_pos += step;
	fl_set_counter_value(stg_up_count, up_polar_pos);
	rotate_polarizer(BOTH_POLARS, step);
}

else
{
	rotate_polarizer(LOWER_POLAR, step);
}
}

/****************************************************************/
/*Crosses the polarizers so they are at 90 degrees to each other*/
/****************************************************************/
void stg_cross_pol_proc(FL_OBJECT *n, long m)
{
stage_command(ROTATING_UPPER_POLAR);
cross_polars(UPPER_POLAR);
fl_show_message("","POLARS ARE NOW CROSSED !","");
fl_set_counter_value(stg_up_count, low_polar_pos);
up_polar_pos = low_polar_pos;
}

/****************************************************************/
/*Zeros counters for polarizers.				*/
/****************************************************************/
void zero_polarizers_proc(FL_OBJECT *n, long m)
{
stage_command(CLEAR_POLARIZER_COUNTERS);
}

/****************************************************************/
/*Changes the upper filter.					*/
/****************************************************************/
void filt_proc(FL_OBJECT *n, long m)
{

if(m == 0)
	stage_command(ROTATING_UPPER_POLAR);
else if(m == 1)
	stage_command(ND_FILTER);
else
	stage_command(FIXED_UPPER_POLAR);
}

/****************************************************************/
/*fixed polar.							*/
/****************************************************************/
void stg_axis_proc(FL_OBJECT *n, long m)
{

if(m == 0 || m == 1)
{
	stage_command(FIXED_UPPER_POLAR);
	cross_polars(BOTH_POLARS);
	
	if(m == 1)
		rotate_polarizer(BOTH_POLARS, 100);

	fl_set_counter_value(stg_up_count, 0);
	up_polar_pos = 0;
	fl_set_counter_value(stg_low_count, 0);
	low_polar_pos = 0;
}

else
{
	fl_set_counter_value(stg_up_count, 0);
	up_polar_pos = 0;
	fl_set_counter_value(stg_low_count, 0);
	low_polar_pos = 0;
}
}

/****************************************************************/
/*exits stage form.						*/
/****************************************************************/
void stg_done_proc(FL_OBJECT *n, long m)
{

fl_hide_all();
fl_show_object(main_grp);
}
