/********************************************************/
/* 	TWOD MODULE					*/
/* 	Provides routines for displaying the 2D info	*/
/*	defined by the multi-spectral analysis.		*/
/********************************************************/
/*      Copyright, 1995                                 */
/*      Ph.D. Frank Fueten                              */
/*      Department of Earth Sciences                    */
/*      Brock University                                */
/*      St. Catharines, Ontario, Canada                 */
/********************************************************/
/*      Ideas      :    Frank Fueten, Scott Goodchild   */
/*      Programming:    Vladimir Tucakov                */
/********************************************************/


#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include <stdlib.h>
#include "image.h"
#include "forms.h"
#include "gv_forms.h"
#include "basic_imm.h"
#include "types.h"
#include "twoD.h"

extern long twoDwin;
extern int xsize,ysize;
extern screenT screen;
extern long pers_win;
extern box_T *boxing;
extern char curr_domain;
extern thetype curr_space;

char two_dump[XSIZE][YSIZE][3];

static long i[2] = {1,1};
static done = FALSE;

/********************************************************/
/* REturns weather the given rgb point is witheen	*/
/* the given single box					*/
/********************************************************/
int in_box(box_T *l_box, short r, short g, short b)
{
if(l_box != NULL &&
                (r>=l_box->r_min) && (r<=l_box->r_max) &&
                (g>=l_box->g_min) && (g<=l_box->g_max) &&
                (b>=l_box->b_min) && (b<=l_box->b_max))
{
	return(TRUE);
}
else
	return(FALSE);
}

/********************************************************/
/* checks weather the given rgb point is witheen any of	*/
/* defined boxes					*/
/********************************************************/
short check_boxes(box_T *box, short r, short g, short b)
{
char temp;

while (box != NULL)
{
	if(in_box(box,r,g,b)==TRUE)
	{
		return((short) box->color);
	}
	else
		box = box->next;
}
return (-1);
}


/********************************************************/
/* Just dumps the given data onto the screen. the	*/
/* procedure is introduce to ensure that the right	*/
/* window is filled with the information		*/
/********************************************************/
void disp_twoD(screenT *orig_data)
{
int l,k;

	winset(twoDwin);
	gconfig();
	lrectwrite(0,0, (orig_data->xsize)-1, (orig_data->ysize)-1,
			(unsigned long *) two_dump);
}

/********************************************************/
/********************************************************/
void indx_alive_proc(FL_OBJECT *n, long m)
{
	prep_twoD(NULL, &screen, boxing, curr_domain, curr_space);
	disp_twoD(&screen);
}


/********************************************************/
/* prepares the "screen" to be displayed depending	*/
/* on the domain, the boxing information and kind of	*/
/* filtering.						*/
/********************************************************/
void prep_twoD(short *old, screenT *orig_data, 
			box_T *box, char dom, thetype space)
{
int l,k;
long count=0;
short colo;
short temp1, temp2, temp3;
short r,g,b;
float ratio;

if (dom == DOM_AVER)
	ratio = (255.0*(float)screen.no_frames)/(float)TOT_AXIS_SIZE;
else
	ratio = 255.0/TOT_AXIS_SIZE;


fprintf(stderr,"%d,%d,%d,%d,%d,%d\n",box->r_min,box->r_max,
			box->g_min,box->g_max,
			box->b_min,box->b_max);
fprintf(stderr,"domain in prep 2D is %d\n",dom);
	for (k=0 ; k<screen.xsize; k++)
	{
		for (l=0; l<screen.ysize; l++)
		{
		if (dom == DOM_AVER)
		{
		    r=(short) (((float)screen.scr[k][l].tot[0])/ratio);
		    g=(short) (((float)screen.scr[k][l].tot[1])/ratio);
		    b=(short) (((float)screen.scr[k][l].tot[2])/ratio);
		}
		else if (dom == DOM_MINV)
		{
		    r=(short) (((float)screen.scr[k][l].min[0])/ratio);
		    g=(short) (((float)screen.scr[k][l].min[1])/ratio);
		    b=(short) (((float)screen.scr[k][l].min[2])/ratio);
		}
		else if (dom == DOM_MINP)
		{
		    r=(short) (((float)screen.scr[k][l].min_pos[0])/ratio);
		    g=(short) (((float)screen.scr[k][l].min_pos[1])/ratio);
		    b=(short) (((float)screen.scr[k][l].min_pos[2])/ratio);
		}
		else if (dom == DOM_MAXV)
		{
		    r=(short) (((float)screen.scr[k][l].max[SR])/ratio);
		    g=(short) (((float)screen.scr[k][l].max[SG])/ratio);
		    b=(short) (((float)screen.scr[k][l].max[SB])/ratio);
		}
		else if (dom == DOM_MAXP)
		{
		    r=(short) (((float)screen.scr[k][l].max_pos[0])/ratio);
		    g=(short) (((float)screen.scr[k][l].max_pos[1])/ratio);
		    b=(short) (((float)screen.scr[k][l].max_pos[2])/ratio);
		}
		    if ((fl_get_button(indx_alive) == 1 &&
			(colo = check_boxes(box, r, g, b)) != -1 &&
			get_from_space(space,r,g,b) != 0) ||

			(fl_get_button(indx_alive) == 0 &&
				(colo = check_boxes(box, r, g, b)) != -1))
			{
				getmcolor((unsigned short)colo,&temp1,
						&temp2,&temp3);
				count++;
				assval(two_dump,k,l,R,(char) temp1);
				assval(two_dump,k,l,G,(char) temp2);
				assval(two_dump,k,l,B,(char) temp3);
				assval(two_dump,k,l,0,0);
			}
			else
			{
				assval(two_dump,k,l,R,0);
				assval(two_dump,k,l,G,0);
				assval(two_dump,k,l,B,0);
				assval(two_dump,k,l,0,0);
			}
		}
	}
fprintf(stderr,"\n\nNUMBER OF POINTS DISPLAYED IN 2D = %d\n\n", count);

}

/********************************************************/
/* Redraws the picture when the "redraw" button is	*/
/* pressed. This is done to avoid the conflict betwee	*/
/* gl and fl events					*/
/********************************************************/
void indx_redraw_proc(FL_OBJECT *n, long m)
{
	disp_twoD(&screen);
	draw_data(curr_domain, TRUE);
}

/********************************************************/
/* Closes the twoD meny and return to the main m.s.	*/
/* menu.						*/
/********************************************************/
void indx_done_proc(FL_OBJECT *n, long m)
{
	winclose(twoDwin);
	close_info_win();
	open_pers();
	reenter_m_ms();
	done = TRUE;
}


/********************************************************/
/* Sets up the gl display and prepares the image to be	*/
/* displayed.						*/
/********************************************************/
void twoD_main(char dom, thetype space)
{
FL_OBJECT *obj;
short val, *twoD_scr;

	done = FALSE;
	fl_hide_all();
	fl_show_object(indx_grp);
	winclose(pers_win);
	prefsize(screen.xsize,screen.ysize);
	twoDwin = winopen("test");
	winset(twoDwin);
	singlebuffer();
	pixmode(PM_TTOB, 1);
	RGBmode();
	gconfig();
open_info_win();
draw_data(dom, TRUE);
	twoD_scr = (short*)calloc(screen.xsize*screen.ysize,sizeof(short));

	fprintf(stderr, "BEFORE PREP \n");
	prep_twoD(twoD_scr, &screen, boxing, dom, space);

	disp_twoD(&screen);

	while (!done)
		fl_do_forms();
}

